from collections.abc import Iterable

from itertools import repeat
from math import sqrt

from ..classes import set_node_attributes
from ..exception import NetworkXError
from ..generators.classic import cycle_graph, empty_graph, path_graph
from ..relabel import relabel_nodes
from ..utils import flatten, nodes_or_number, pairwise

__all__ = [
    "grid_2d_graph",
    "grid_graph",
    "hypercube_graph",
    "triangular_lattice_graph",
    "hexagonal_lattice_graph",
]

def grid_2d_graph(m, n, periodic: bool | Iterable = False, create_using=None): ...
def grid_graph(dim, periodic: bool | Iterable = False): ...
def hypercube_graph(n: int): ...
def triangular_lattice_graph(
    m: int, n: int, periodic=False, with_positions=True, create_using=None
): ...
def hexagonal_lattice_graph(
    m: int, n: int, periodic: bool = False, with_positions=True, create_using=None
): ...



from typing import Mapping

from collections import Counter, defaultdict
from hashlib import blake2b
from ..classes.graph import Graph

__all__ = ["weisfeiler_lehman_graph_hash", "weisfeiler_lehman_subgraph_hashes"]

def weisfeiler_lehman_graph_hash(
    G: Graph,
    edge_attr: str | None = None,
    node_attr: str | None = None,
    iterations: int = 3,
    digest_size: int = 16,
) -> str: ...
def weisfeiler_lehman_subgraph_hashes(
    G: Graph,
    edge_attr: str | None = None,
    node_attr: str | None = None,
    iterations: int = 3,
    digest_size: int = 16,
) -> Mapping: ...



from typing import Mapping
from itertools import combinations

from ...classes.graph import Graph

__all__ = ["dispersion"]

def dispersion(
    G: Graph, u=None, v=None, normalized: bool = True, alpha=1.0, b=0.0, c=0.0
) -> Mapping: ...



from typing import Callable
from numpy.random import RandomState
from ..pairwise import (
    pairwise_distances_chunked as pairwise_distances_chunked,
    pairwise_distances as pairwise_distances,
)
from ...preprocessing import LabelEncoder as LabelEncoder
from ...utils import check_random_state as check_random_state, check_X_y as check_X_y
from ..._typing import Int, MatrixLike, ArrayLike, Float

# Authors: Robert Layton <robertlayton@gmail.com>
#          Arnaud Fouchet <foucheta@gmail.com>
#          Thierry Guillemot <thierry.guillemot.work@gmail.com>
# License: BSD 3 clause


import functools

import numpy as np


def check_number_of_labels(n_labels: Int, n_samples: Int) -> None:
    ...


def silhouette_score(
    X: MatrixLike,
    labels: ArrayLike,
    *,
    metric: str | Callable = "euclidean",
    sample_size: None | Int = None,
    random_state: RandomState | None | Int = None,
    **kwds
) -> Float:
    ...


def silhouette_samples(
    X: MatrixLike, labels: ArrayLike, *, metric: str | Callable = "euclidean", **kwds
) -> ArrayLike:
    ...


def calinski_harabasz_score(X: MatrixLike, labels: ArrayLike) -> float:
    ...


def davies_bouldin_score(X: MatrixLike, labels: ArrayLike) -> float:
    ...




import numpy as np


class SequentialDataset32():
    current_index: int
    index: np.ndarray
    index_data_ptr: int
    n_samples: int
    seed: int

    def shuffle(self, seed: int) -> None: ...
    def next(self, x_data_ptr, x_ind_ptr, nnz, y, sample_weight) -> None: ...
    def random(self, x_data_ptr, x_ind_ptr, nnz, y, sample_weight) -> int: ...

    
class SequentialDataset64():
    current_index: int
    index: np.ndarray
    index_data_ptr: int
    n_samples: int
    seed: int

    def shuffle(self, seed: int) -> None: ...
    def next(self, x_data_ptr, x_ind_ptr, nnz, y, sample_weight) -> None: ...
    def random(self, x_data_ptr, x_ind_ptr, nnz, y, sample_weight) -> int: ...


class ArrayDataset32(SequentialDataset32):
    def __init__(self, X, Y, sample_weights, seed=1) -> None: ...

    
class ArrayDataset64(SequentialDataset64):
    def __init__(self, X, Y, sample_weights, seed=1) -> None: ...


class CSRDataset64(SequentialDataset64):
    def __init__(self, X_data, X_indptr, X_indices, Y, sample_weights, seed=1) -> None: ...

    
class CSRDataset32(SequentialDataset32):
    def __init__(self, X_data, X_indptr, X_indices, Y, sample_weights, seed=1) -> None: ...




from .font_manager import FontProperties

family_punc = ...
family_unescape = ...
family_escape = ...
value_punc = ...
value_unescape = ...
value_escape = ...

class FontconfigPatternParser:
    def __init__(self) -> None: ...
    def parse(self, pattern: str) -> dict[str, list[str]]: ...

parse_fontconfig_pattern = ...

def generate_fontconfig_pattern(d: FontProperties) -> str: ...



from typing import Callable, Iterable, Literal, Mapping
from numpy.random import RandomState
from ..base import BaseEstimator
from ..exceptions import FitFailedWarning as FitFailedWarning
from ._split import BaseShuffleSplit
from joblib import logger as logger
from ..metrics import check_scoring as check_scoring
from ..preprocessing import LabelEncoder as LabelEncoder
from contextlib import suppress as suppress
from ._split import check_cv as check_cv
from ..svm._classes import SVC, LinearSVC
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from traceback import format_exc as format_exc
from collections import Counter as Counter
from numpy import ndarray
from functools import partial as partial
from ..base import is_classifier as is_classifier, clone as clone
from ..utils import indexable as indexable, check_random_state as check_random_state
from .._typing import MatrixLike, ArrayLike, Int, Float
from . import BaseCrossValidator

# Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#         Gael Varoquaux <gael.varoquaux@normalesup.org>
#         Olivier Grisel <olivier.grisel@ensta.org>
#         Raghav RV <rvraghav93@gmail.com>
#         Michal Karbownik <michakarbownik@gmail.com>
# License: BSD 3 clause


import warnings
import numbers
import time

import numpy as np
import scipy.sparse as sp


__all__ = [
    "cross_validate",
    "cross_val_score",
    "cross_val_predict",
    "permutation_test_score",
    "learning_curve",
    "validation_curve",
]


def cross_validate(
    estimator: BaseEstimator,
    X: MatrixLike,
    y: None | MatrixLike | ArrayLike = None,
    *,
    groups: None | ArrayLike = None,
    scoring: ArrayLike | None | tuple | Callable | Mapping | str = None,
    cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
    n_jobs: None | Int = None,
    verbose: Int = 0,
    fit_params: None | dict = None,
    pre_dispatch: str | Int = "2*n_jobs",
    return_train_score: bool = False,
    return_estimator: bool = False,
    error_score: str | Float = ...,
):
    ...


def cross_val_score(
    estimator: BaseEstimator,
    X: MatrixLike,
    y: None | MatrixLike | ArrayLike = None,
    *,
    groups: None | ArrayLike = None,
    scoring: None | str | Callable = None,
    cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
    n_jobs: None | Int = None,
    verbose: Int = 0,
    fit_params: None | dict = None,
    pre_dispatch: str | Int = "2*n_jobs",
    error_score: str | Float = ...,
) -> ndarray:
    ...


def cross_val_predict(
    estimator: BaseEstimator,
    X: MatrixLike,
    y: None | MatrixLike | ArrayLike = None,
    *,
    groups: None | ArrayLike = None,
    cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
    n_jobs: None | Int = None,
    verbose: Int = 0,
    fit_params: None | dict = None,
    pre_dispatch: str | Int = "2*n_jobs",
    method: Literal[
        "predict", "predict_proba", "predict_log_proba", "decision_function", "predict"
    ] = "predict",
) -> ndarray:
    ...


def permutation_test_score(
    estimator: BaseEstimator | SVC,
    X: MatrixLike,
    y: None | MatrixLike | ArrayLike,
    *,
    groups: None | ArrayLike = None,
    cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
    n_permutations: Int = 100,
    n_jobs: None | Int = None,
    random_state: RandomState | None | Int = 0,
    verbose: Int = 0,
    scoring: None | str | Callable = None,
    fit_params: None | dict = None,
) -> tuple[Float, ndarray, Float] | tuple[float, ndarray, float]:
    ...


def learning_curve(
    estimator,
    X: MatrixLike,
    y: MatrixLike | ArrayLike,
    *,
    groups: None | ArrayLike = None,
    train_sizes: ArrayLike = ...,
    cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
    scoring: None | str | Callable = None,
    exploit_incremental_learning: bool = False,
    n_jobs: None | Int = None,
    pre_dispatch: str | Int = "all",
    verbose: Int = 0,
    shuffle: bool = False,
    random_state: RandomState | None | Int = None,
    error_score: str | Float = ...,
    return_times: bool = False,
    fit_params: None | dict = None,
) -> tuple[ndarray, ndarray, ndarray, ndarray, ndarray]:
    ...


def validation_curve(
    estimator: SVC | LinearSVC,
    X: MatrixLike,
    y: None | MatrixLike | ArrayLike,
    *,
    param_name: str,
    param_range: ArrayLike,
    groups: None | ArrayLike = None,
    cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
    scoring: None | str | Callable = None,
    n_jobs: None | Int = None,
    pre_dispatch: str | Int = "all",
    verbose: Int = 0,
    error_score: str | Float = ...,
    fit_params: None | dict = None,
) -> tuple[ndarray, ndarray]:
    ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
import re
from .. import glsl
from ..util import logger

def remove_comments(code): ...
def merge_includes(code): ...
def preprocess(code): ...



__all__ = ["polygon_clip", "polygon_area"]

import numpy as np

from .version_requirements import require

@require("matplotlib", ">=3.0.3")
def polygon_clip(rp, cp, r0, c0, r1, c1): ...
def polygon_area(pr, pc) -> float: ...



from typing import Any, ClassVar, Literal, TypeVar
from collections import Counter as Counter
from scipy import sparse as sp
from numpy import ndarray
from ..utils._param_validation import StrOptions as StrOptions, Hidden as Hidden
from ..base import BaseEstimator, TransformerMixin
from scipy.sparse import spmatrix
from .._typing import Int, MatrixLike, ArrayLike
from ..utils import is_scalar_nan as is_scalar_nan
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    FLOAT_DTYPES as FLOAT_DTYPES,
)

SimpleImputer_Self = TypeVar("SimpleImputer_Self", bound="SimpleImputer")
MissingIndicator_Self = TypeVar("MissingIndicator_Self", bound="MissingIndicator")

# Authors: Nicolas Tresegnie <nicolas.tresegnie@gmail.com>
#          Sergey Feldman <sergeyfeldman@gmail.com>
# License: BSD 3 clause

import numbers
import warnings

import numpy as np
import numpy.ma as ma


class _BaseImputer(TransformerMixin, BaseEstimator):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        missing_values=...,
        add_indicator: bool = False,
        keep_empty_features: bool = False,
    ) -> None:
        ...


class SimpleImputer(_BaseImputer):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    indicator_: MissingIndicator = ...
    statistics_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        missing_values: float | None | str | int = ...,
        strategy: str = "mean",
        fill_value: float | None | str | int = None,
        verbose: str | Int = "deprecated",
        copy: bool = True,
        add_indicator: bool = False,
        keep_empty_features: bool = False,
    ) -> None:
        ...

    def fit(
        self: SimpleImputer_Self, X: MatrixLike, y: Any = None
    ) -> SimpleImputer_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...


class MissingIndicator(TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    features_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        missing_values: float | None | str | int = ...,
        features: Literal["missing-only", "all", "missing-only"] = "missing-only",
        sparse: Literal["auto", "auto"] | bool = "auto",
        error_on_new: bool = True,
    ) -> None:
        ...

    def fit(
        self: MissingIndicator_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> MissingIndicator_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...

    def fit_transform(
        self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> ndarray | spmatrix:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...



from numpy.typing import ArrayLike
from typing import Literal
import os
import sys

def _show_skimage_info(): ...

class PytestTester(object):
    def __init__(self, module_name): ...
    def __call__(
        self,
        label: Literal["fast", "full"] = "fast",
        verbose: int = 1,
        extra_argv: ArrayLike | None = None,
        doctests: bool = False,
        coverage: bool = False,
        durations: int = ...,
        tests=None,
    ) -> bool: ...



from .betweenness import (
    betweenness_centrality as betweenness_centrality,
    edge_betweenness_centrality as edge_betweenness_centrality,
    edge_betweenness as edge_betweenness,
)
from .betweenness_subset import (
    betweenness_centrality_subset as betweenness_centrality_subset,
    betweenness_centrality_source as betweenness_centrality_source,
    edge_betweenness_centrality_subset as edge_betweenness_centrality_subset,
)
from .closeness import (
    closeness_centrality as closeness_centrality,
    incremental_closeness_centrality as incremental_closeness_centrality,
)
from .current_flow_betweenness import (
    current_flow_betweenness_centrality as current_flow_betweenness_centrality,
    approximate_current_flow_betweenness_centrality as approximate_current_flow_betweenness_centrality,
    edge_current_flow_betweenness_centrality as edge_current_flow_betweenness_centrality,
)
from .current_flow_betweenness_subset import (
    current_flow_betweenness_centrality_subset as current_flow_betweenness_centrality_subset,
    edge_current_flow_betweenness_centrality_subset as edge_current_flow_betweenness_centrality_subset,
)
from .current_flow_closeness import (
    current_flow_closeness_centrality as current_flow_closeness_centrality,
    information_centrality as information_centrality,
)
from .degree_alg import (
    degree_centrality as degree_centrality,
    in_degree_centrality as in_degree_centrality,
    out_degree_centrality as out_degree_centrality,
)
from .dispersion import dispersion as dispersion
from .eigenvector import (
    eigenvector_centrality as eigenvector_centrality,
    eigenvector_centrality_numpy as eigenvector_centrality_numpy,
)
from .group import (
    group_betweenness_centrality as group_betweenness_centrality,
    group_closeness_centrality as group_closeness_centrality,
    group_degree_centrality as group_degree_centrality,
    group_in_degree_centrality as group_in_degree_centrality,
    group_out_degree_centrality as group_out_degree_centrality,
    prominent_group as prominent_group,
)
from .harmonic import harmonic_centrality as harmonic_centrality
from .katz import (
    katz_centrality as katz_centrality,
    katz_centrality_numpy as katz_centrality_numpy,
)
from .load import (
    load_centrality as load_centrality,
    edge_load_centrality as edge_load_centrality,
)
from .percolation import (
    percolation_centrality as percolation_centrality,
)
from .reaching import (
    global_reaching_centrality as global_reaching_centrality,
    local_reaching_centrality as local_reaching_centrality,
)
from .second_order import second_order_centrality as second_order_centrality
from .subgraph_alg import (
    subgraph_centrality_exp as subgraph_centrality_exp,
    subgraph_centrality as subgraph_centrality,
    communicability_betweenness_centrality as communicability_betweenness_centrality,
    estrada_index as estrada_index,
)
from .trophic import (
    trophic_levels as trophic_levels,
    trophic_differences as trophic_differences,
    trophic_incoherence_parameter as trophic_incoherence_parameter,
)
from .voterank_alg import voterank as voterank



"""
This type stub file was generated by pyright.
"""

import tokenize
from codeop import CommandCompiler, Compile
from typing import List, Tuple

"""Input transformer machinery to support IPython special syntax.

This includes the machinery to recognise and transform ``%magic`` commands,
``!system`` commands, ``help?`` querying, prompt stripping, and so forth.

Added: IPython 7.0. Replaces inputsplitter and inputtransformer which were
deprecated in 7.0.
"""
_indent_re = ...
def leading_empty_lines(lines):
    """Remove leading empty lines

    If the leading lines are empty or contain only whitespace, they will be
    removed.
    """
    ...

def leading_indent(lines):
    """Remove leading indentation.

    If the first line starts with a spaces or tabs, the same whitespace will be
    removed from each following line in the cell.
    """
    ...

class PromptStripper:
    """Remove matching input prompts from a block of input.

    Parameters
    ----------
    prompt_re : regular expression
        A regular expression matching any input prompt (including continuation,
        e.g. ``...``)
    initial_re : regular expression, optional
        A regular expression matching only the initial prompt, but not continuation.
        If no initial expression is given, prompt_re will be used everywhere.
        Used mainly for plain Python prompts (``>>>``), where the continuation prompt
        ``...`` is a valid Python expression in Python 3, so shouldn't be stripped.

    Notes
    -----

    If initial_re and prompt_re differ,
    only initial_re will be tested against the first line.
    If any prompt is found on the first two lines,
    prompts will be stripped from the rest of the block.
    """
    def __init__(self, prompt_re, initial_re=...) -> None:
        ...
    
    def __call__(self, lines):
        ...
    


classic_prompt = ...
ipython_prompt = ...
def cell_magic(lines):
    ...

def find_end_of_continued_line(lines, start_line: int):
    """Find the last line of a line explicitly extended using backslashes.

    Uses 0-indexed line numbers.
    """
    ...

def assemble_continued_line(lines, start: Tuple[int, int], end_line: int):
    r"""Assemble a single line from multiple continued line pieces

    Continued lines are lines ending in ``\``, and the line following the last
    ``\`` in the block.

    For example, this code continues over multiple lines::

        if (assign_ix is not None) \
             and (len(line) >= assign_ix + 2) \
             and (line[assign_ix+1].string == '%') \
             and (line[assign_ix+2].type == tokenize.NAME):

    This statement contains four continued line pieces.
    Assembling these pieces into a single line would give::

        if (assign_ix is not None) and (len(line) >= assign_ix + 2) and (line[...

    This uses 0-indexed line numbers. *start* is (lineno, colno).

    Used to allow ``%magic`` and ``!system`` commands to be continued over
    multiple lines.
    """
    ...

class TokenTransformBase:
    """Base class for transformations which examine tokens.

    Special syntax should not be transformed when it occurs inside strings or
    comments. This is hard to reliably avoid with regexes. The solution is to
    tokenise the code as Python, and recognise the special syntax in the tokens.

    IPython's special syntax is not valid Python syntax, so tokenising may go
    wrong after the special syntax starts. These classes therefore find and
    transform *one* instance of special syntax at a time into regular Python
    syntax. After each transformation, tokens are regenerated to find the next
    piece of special syntax.

    Subclasses need to implement one class method (find)
    and one regular method (transform).

    The priority attribute can select which transformation to apply if multiple
    transformers match in the same place. Lower numbers have higher priority.
    This allows "%magic?" to be turned into a help call rather than a magic call.
    """
    priority = ...
    def sortby(self):
        ...
    
    def __init__(self, start) -> None:
        ...
    
    @classmethod
    def find(cls, tokens_by_line):
        """Find one instance of special syntax in the provided tokens.

        Tokens are grouped into logical lines for convenience,
        so it is easy to e.g. look at the first token of each line.
        *tokens_by_line* is a list of lists of tokenize.TokenInfo objects.

        This should return an instance of its class, pointing to the start
        position it has found, or None if it found no match.
        """
        ...
    
    def transform(self, lines: List[str]):
        """Transform one instance of special syntax found by ``find()``

        Takes a list of strings representing physical lines,
        returns a similar list of transformed lines.
        """
        ...
    


class MagicAssign(TokenTransformBase):
    """Transformer for assignments from magics (a = %foo)"""
    @classmethod
    def find(cls, tokens_by_line): # -> None:
        """Find the first magic assignment (a = %foo) in the cell.
        """
        ...
    
    def transform(self, lines: List[str]):
        """Transform a magic assignment found by the ``find()`` classmethod.
        """
        ...
    


class SystemAssign(TokenTransformBase):
    """Transformer for assignments from system commands (a = !foo)"""
    @classmethod
    def find(cls, tokens_by_line): # -> None:
        """Find the first system assignment (a = !foo) in the cell.
        """
        ...
    
    def transform(self, lines: List[str]):
        """Transform a system assignment found by the ``find()`` classmethod.
        """
        ...
    


ESC_SHELL = ...
ESC_SH_CAP = ...
ESC_HELP = ...
ESC_HELP2 = ...
ESC_MAGIC = ...
ESC_MAGIC2 = ...
ESC_QUOTE = ...
ESC_QUOTE2 = ...
ESC_PAREN = ...
ESCAPE_SINGLES = ...
ESCAPE_DOUBLES = ...
tr = ...
class EscapedCommand(TokenTransformBase):
    """Transformer for escaped commands like %foo, !foo, or /foo"""
    @classmethod
    def find(cls, tokens_by_line): # -> None:
        """Find the first escaped command (%foo, !foo, etc.) in the cell.
        """
        ...
    
    def transform(self, lines):
        """Transform an escaped line found by the ``find()`` classmethod.
        """
        ...
    


_help_end_re = ...
class HelpEnd(TokenTransformBase):
    """Transformer for help syntax: obj? and obj??"""
    priority = ...
    def __init__(self, start, q_locn) -> None:
        ...
    
    @classmethod
    def find(cls, tokens_by_line): # -> None:
        """Find the first help command (foo?) in the cell.
        """
        ...
    
    def transform(self, lines):
        """Transform a help command found by the ``find()`` classmethod.
        """
        ...
    


def make_tokens_by_line(lines: List[str]):
    """Tokenize a series of lines and group tokens by line.

    The tokens for a multiline Python string or expression are grouped as one
    line. All lines except the last lines should keep their line ending ('\\n',
    '\\r\\n') for this to properly work. Use `.splitlines(keeplineending=True)`
    for example when passing block of text to this function.

    """
    ...

def has_sunken_brackets(tokens: List[tokenize.TokenInfo]):
    """Check if the depth of brackets in the list of tokens drops below 0"""
    ...

def show_linewise_tokens(s: str): # -> None:
    """For investigation and debugging"""
    ...

TRANSFORM_LOOP_LIMIT = ...
class TransformerManager:
    """Applies various transformations to a cell or code block.

    The key methods for external use are ``transform_cell()``
    and ``check_complete()``.
    """
    def __init__(self) -> None:
        ...
    
    def do_one_token_transform(self, lines):
        """Find and run the transform earliest in the code.

        Returns (changed, lines).

        This method is called repeatedly until changed is False, indicating
        that all available transformations are complete.

        The tokens following IPython special syntax might not be valid, so
        the transformed code is retokenised every time to identify the next
        piece of special syntax. Hopefully long code cells are mostly valid
        Python, not using lots of IPython special syntax, so this shouldn't be
        a performance issue.
        """
        ...
    
    def do_token_transforms(self, lines):
        ...
    
    def transform_cell(self, cell: str) -> str:
        """Transforms a cell of input code"""
        ...
    
    def check_complete(self, cell: str):
        """Return whether a block of code is ready to execute, or should be continued

        Parameters
        ----------
        cell : string
            Python input code, which can be multiline.

        Returns
        -------
        status : str
            One of 'complete', 'incomplete', or 'invalid' if source is not a
            prefix of valid code.
        indent_spaces : int or None
            The number of spaces by which to indent the next line of code. If
            status is not 'incomplete', this is None.
        """
        ...
    


def find_last_indent(lines):
    ...

class MaybeAsyncCompile(Compile):
    def __init__(self, extra_flags=...) -> None:
        ...
    


class MaybeAsyncCommandCompiler(CommandCompiler):
    def __init__(self, extra_flags=...) -> None:
        ...
    


_extra_flags = ...
compile_command = ...



from collections.abc import Iterable
from ...classes.graph import Graph
from collections import Counter, defaultdict

from ...classes.graph import Graph
from ...utils import groups, not_implemented_for, py_random_state

__all__ = ["label_propagation_communities", "asyn_lpa_communities"]

@py_random_state(2)
def asyn_lpa_communities(
    G: Graph, weight: str | None = None, seed=None
) -> Iterable: ...
def label_propagation_communities(G: Graph) -> Iterable: ...



from skimage.future.graph.rag import RAG
from typing import Mapping, Callable, Sequence, Literal
import networkx as nx
import numpy as np
from numpy.lib.stride_tricks import as_strided
from scipy import ndimage as ndi
from scipy import sparse
import math

from ..._shared.version_requirements import require

def _edge_generator_from_csr(csr_matrix): ...
def min_weight(graph: RAG, src: int, dst: int, n: int) -> Mapping: ...
def _add_edge_filter(values, graph): ...

class RAG(nx.Graph):
    def __init__(self, label_image=None, connectivity=1, data=None, **attr): ...
    def merge_nodes(
        self,
        src: int,
        dst: int,
        weight_func: Callable = ...,
        in_place: bool = True,
        extra_arguments: Sequence = [],
        extra_keywords: Mapping = {},
    ) -> int: ...
    def add_node(self, n, attr_dict=None, **attr): ...
    def add_edge(self, u, v, attr_dict=None, **attr): ...
    def copy(self): ...
    def fresh_copy(self): ...
    def next_id(self) -> int: ...
    def _add_node_silent(self, n): ...

def rag_mean_color(
    image,
    labels,
    connectivity: int = 2,
    mode: Literal["distance", "similarity"] = "distance",
    sigma: float = 255.0,
) -> RAG: ...
def rag_boundary(labels, edge_map, connectivity=2): ...
@require("matplotlib", ">=3.0.3")
def show_rag(
    labels,
    rag: RAG,
    image,
    border_color="black",
    edge_width: float = 1.5,
    edge_cmap="magma",
    img_cmap="bone",
    in_place: bool = True,
    ax=None,
): ...



from collections.abc import Iterable
from itertools import chain

from ..classes.graph import Graph
from ..utils import arbitrary_element

__all__ = ["dominating_set", "is_dominating_set"]

def dominating_set(G: Graph, start_with=None) -> set: ...
def is_dominating_set(G: Graph, nbunch: Iterable): ...



from typing import Any, Callable, NamedTuple
from ._typing import *
from .backend_bases import RendererBase, MouseEvent
from .patches import Circle, Patch, Rectangle, RegularPolygon
from .figure import Figure
from .transforms import Bbox, Transform
from .axes import Axes
from .path import Path
from .patheffects import AbstractPathEffect

def allow_rasterization(
    draw,
)-> Callable: ...

class _Unset:
    def __repr__(self) -> str: ...

class Artist:

    zorder: int = ...
    def __init_subclass__(cls)-> None: ...
    def __init__(self) -> None: ...
    def __getstate__(self)-> dict: ...
    def remove(self)-> None: ...
    def have_units(self)-> bool: ...
    def convert_xunits(self, x: int) -> int: ...
    def convert_yunits(self, y: int) -> int: ...
    @property
    def axes(self) -> Axes: ...
    @axes.setter
    def axes(self, new_axes: Axes): ...
    @property
    def stale(self)-> bool: ...
    @stale.setter
    def stale(self, val: bool): ...
    def get_window_extent(self, renderer: RendererBase = ...)-> Bbox: ...
    def get_tightbbox(self, renderer: RendererBase = ...) -> Bbox: ...
    def add_callback(self, func: Callable) -> int: ...
    def remove_callback(self, oid: int)-> None: ...
    def pchanged(self) -> None: ...
    def is_transform_set(self) -> bool: ...
    def set_transform(self, t: Transform) -> None: ...
    def get_transform(self) -> Transform: ...
    def get_children(self) -> list[Artist]: ...
    def contains(self, mouseevent: MouseEvent): ...
    def pickable(self) -> bool: ...
    def pick(self, mouseevent: MouseEvent)-> None: ...
    def set_picker(self, picker: bool) -> None: ...
    def get_picker(self) -> bool: ...
    def get_url(self) -> str: ...
    def set_url(self, url: str)-> None: ...
    def get_gid(self) -> str | None: ...
    def set_gid(self, gid: str) -> None: ...
    def get_snap(self) -> bool | None: ...
    def set_snap(self, snap: bool | None)-> None: ...
    def get_sketch_params(self) -> None | tuple[float, float, float]: ...
    def set_sketch_params(
        self, scale: float = ..., length: float = 128, randomness: float = 16
    )-> None: ...
    def set_path_effects(self, path_effects: AbstractPathEffect)-> None: ...
    def get_path_effects(self) -> list[AbstractPathEffect]: ...
    def get_figure(self) -> Figure: ...
    def set_figure(self, fig: Figure)-> None: ...
    def set_clip_box(self, clipbox: Bbox)-> None: ...
    def set_clip_path(
        self,
        path: Circle | RegularPolygon | Rectangle,
        transform: Transform | None = ...,
    ) -> None: ...
    def get_alpha(self) -> float: ...
    def get_visible(self) -> bool: ...
    def get_animated(self) -> bool: ...
    def get_in_layout(self) -> bool: ...
    def get_clip_on(self) -> bool: ...
    def get_clip_box(self) -> Bbox: ...
    def get_clip_path(self) -> Path: ...
    def get_transformed_clip_path_and_affine(self): ...
    def set_clip_on(self, b: bool) -> None: ...
    def get_rasterized(self) -> bool: ...
    def set_rasterized(self, rasterized: bool) -> None: ...
    def get_agg_filter(self) -> None | Callable: ...
    def set_agg_filter(self, filter_func: Callable) -> None: ...
    def draw(self, renderer: RendererBase)-> None: ...
    def set_alpha(self, alpha: float | None) -> None: ...
    def set_visible(self, b: bool)-> None: ...
    def set_animated(self, b: bool)-> None: ...
    def set_in_layout(self, in_layout: bool)-> None: ...
    def get_label(self) -> str: ...
    def set_label(self, s: str) -> None: ...
    def get_zorder(self) -> int: ...
    def set_zorder(self, level: float) -> None: ...
    @property
    def sticky_edges(self) -> NamedTuple: ...
    def update_from(self, other: Patch) -> None: ...
    def properties(self)-> dict: ...
    def update(self, props: dict) -> list: ...
    def set(self, **kwargs) -> list | None: ...
    def findobj(self, match: Callable = ..., include_self: bool = ...) -> list: ...
    def get_cursor_data(self, event: MouseEvent)-> None: ...
    def format_cursor_data(self, data) -> str: ...
    def get_mouseover(self) -> bool: ...
    def set_mouseover(self, mouseover: bool) -> None: ...
    mouseover: property = ...

class ArtistInspector:
    def __init__(self, o: Rectangle) -> None: ...
    def get_aliases(self)-> dict: ...
    def get_valid_values(self, attr: str) -> str: ...
    def get_setters(self) -> list[str]: ...
    def is_alias(self, o)-> bool: ...
    def aliased_name(self, s: str) -> str: ...
    def aliased_name_rest(self, s: str, target)-> str: ...
    def pprint_setters(self, prop: str | None = ..., leadingspace=...) -> str|list[str]: ...
    def pprint_setters_rest(self, prop: str | None = ..., leadingspace=...)-> str|list[str]: ...
    def properties(self) -> dict[str, Any]: ...
    def pprint_getters(self) -> list[str]: ...

def getp(obj: Artist, property: str | None = ...): ...

get = getp

def setp(obj: Artist | list, *args, file: FileLike = ..., **kwargs) -> list|None: ...
def kwdoc(artist: Artist) -> str: ...
def _finalize_rasterization(draw): ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
from .raw_path_collection import RawPathCollection
from .agg_path_collection import AggPathCollection
from .agg_fast_path_collection import AggFastPathCollection

def PathCollection(mode="agg", *args, **kwargs): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.stats.mvn, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def dkblck() -> typing.Any:
    "'i'-scalar\n"
    ...

def mvndst(lower, upper, infin, correl, maxpts=..., abseps=..., releps=...) -> typing.Any:
    "error,value,inform = mvndst(lower,upper,infin,correl,[maxpts,abseps,releps])\n\nWrapper for ``mvndst``.\n\nParameters\n----------\nlower : input rank-1 array('d') with bounds (n)\nupper : input rank-1 array('d') with bounds (n)\ninfin : input rank-1 array('i') with bounds (n)\ncorrel : input rank-1 array('d') with bounds (n*(n-1)/2)\n\nOther Parameters\n----------------\nmaxpts : input int, optional\n    Default: 2000\nabseps : input float, optional\n    Default: 1e-06\nreleps : input float, optional\n    Default: 1e-06\n\nReturns\n-------\nerror : float\nvalue : float\ninform : int\n"
    ...

def mvnun(lower, upper, means, covar, maxpts=..., abseps=..., releps=...) -> typing.Any:
    "value,inform = mvnun(lower,upper,means,covar,[maxpts,abseps,releps])\n\nWrapper for ``mvnun``.\n\nParameters\n----------\nlower : input rank-1 array('d') with bounds (d)\nupper : input rank-1 array('d') with bounds (d)\nmeans : input rank-2 array('d') with bounds (d,n)\ncovar : input rank-2 array('d') with bounds (d,d)\n\nOther Parameters\n----------------\nmaxpts : input int, optional\n    Default: d*1000\nabseps : input float, optional\n    Default: 1e-06\nreleps : input float, optional\n    Default: 1e-06\n\nReturns\n-------\nvalue : float\ninform : int\n"
    ...

def mvnun_weighted(lower, upper, means, weights, covar, maxpts=..., abseps=..., releps=...) -> typing.Any:
    "value,inform = mvnun_weighted(lower,upper,means,weights,covar,[maxpts,abseps,releps])\n\nWrapper for ``mvnun_weighted``.\n\nParameters\n----------\nlower : input rank-1 array('d') with bounds (d)\nupper : input rank-1 array('d') with bounds (d)\nmeans : input rank-2 array('d') with bounds (d,n)\nweights : input rank-1 array('d') with bounds (n)\ncovar : input rank-2 array('d') with bounds (d,d)\n\nOther Parameters\n----------------\nmaxpts : input int, optional\n    Default: d*1000\nabseps : input float, optional\n    Default: 1e-06\nreleps : input float, optional\n    Default: 1e-06\n\nReturns\n-------\nvalue : float\ninform : int\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from vispy.util.svg.color import Color
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from functools import lru_cache

import numpy as np

from ... import gloo, glsl
from ...color import Color, ColorArray, get_colormap
from ..shaders import Function
from ..visual import Visual, CompoundVisual
from ...util.profiler import Profiler

from .dash_atlas import DashAtlas

joins: dict = ...

caps: dict = ...

class LineVisual(CompoundVisual):

    _join_types = ...

    _cap_types = ...

    def __init__(
        self,
        pos: ArrayLike | None = None,
        color: Color | tuple | ArrayLike = ...,
        width=1,
        connect: str | ArrayLike = "strip",
        method: str = "gl",
        antialias: bool = False,
    ): ...
    @property
    def join_types(self): ...
    @property
    def cap_types(self): ...
    @property
    def antialias(self): ...
    @antialias.setter
    def antialias(self, aa): ...
    @property
    def method(self): ...
    @method.setter
    def method(self, method): ...
    def set_data(
        self,
        pos: ArrayLike | None = None,
        color: Color | tuple | ArrayLike | None = None,
        width=None,
        connect: str | ArrayLike | None = None,
    ): ...
    @property
    def color(self): ...
    @property
    def width(self): ...
    @property
    def connect(self): ...
    @property
    def pos(self): ...
    def _interpret_connect(self): ...
    def _interpret_color(self, color_in=None): ...
    def _compute_bounds(self, axis, view): ...
    def _prepare_draw(self, view): ...

class _GLLineVisual(Visual):
    _shaders: dict = ...

    def __init__(self, parent): ...
    @staticmethod
    @lru_cache(maxsize=2)
    def _ensure_vec4_func(dims): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...

class _AggLineVisual(Visual):
    _agg_vtype = ...

    _shaders: dict = ...

    def __init__(self, parent): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...
    @classmethod
    def _agg_bake(cls, vertices, color, closed=False): ...



import sys

from packaging import version as _version

def ensure_python_version(min_version): ...
def _check_version(actver, version, cmp_op): ...
def get_module_version(module_name): ...
def is_installed(name: str, version: str | None = None) -> bool: ...
def require(name: str, version: str | None = None): ...
def get_module(module_name: str, version: str | None = None): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.csgraph._flow, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import scipy.sparse.csr as _mod_scipy_sparse_csr

DTYPE = _mod_numpy.float64
ITYPE = _mod_numpy.int32
class MaximumFlowResult(_mod_builtins.object):
    'Represents the result of a maximum flow calculation.\n\n    Attributes\n    ----------\n    flow_value : int\n        The value of the maximum flow.\n    residual : csr_matrix\n        The residual graph with respect to the maximum flow.\n    '
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, flow_value, residual) -> None:
        'Represents the result of a maximum flow calculation.\n\n    Attributes\n    ----------\n    flow_value : int\n        The value of the maximum flow.\n    residual : csr_matrix\n        The residual graph with respect to the maximum flow.\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __module__: str
    def __repr__(self) -> str:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def __weakref__(self) -> typing.Any:
        'list of weak references to the object (if defined)'
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _add_reverse_edges() -> typing.Any:
    'Add reversed edges to all edges in a graph.\n\n    This adds to a given directed weighted graph all edges in the reverse\n    direction and give them weight 0, unless they already exist.\n\n    Parameters\n    ----------\n    a : csr_matrix\n        The square matrix in CSR format representing a directed graph\n\n    Returns\n    -------\n    res : csr_matrix\n        A new matrix in CSR format in which the missing edges are represented\n        by explicit zeros.\n\n    '
    ...

def _make_edge_pointers() -> typing.Any:
    'Create for each edge pointers to its reverse and its tail.'
    ...

csr_matrix = _mod_scipy_sparse_csr.csr_matrix
def isspmatrix_csr(x) -> typing.Any:
    'Is x of csr_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csr matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csr matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix_csr\n    >>> isspmatrix_csr(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csr(csc_matrix([[5]]))\n    False\n    '
    ...

def maximum_flow(csgraph, source, sink) -> typing.Any:
    "\n    maximum_flow(csgraph, source, sink)\n\n    Maximize the flow between two vertices in a graph.\n\n    .. versionadded:: 1.4.0\n\n    Parameters\n    ----------\n    csgraph : csr_matrix\n        The square matrix representing a directed graph whose (i, j)'th entry\n        is an integer representing the capacity of the edge between\n        vertices i and j.\n    source : int\n        The source vertex from which the flow flows.\n    sink : int\n        The sink vertex to which the flow flows.\n\n    Returns\n    -------\n    res : MaximumFlowResult\n        A maximum flow represented by a ``MaximumFlowResult``\n        which includes the value of the flow in ``flow_value``,\n        and the residual graph in ``residual``.\n\n    Raises\n    ------\n    TypeError:\n        if the input graph is not in CSR format.\n\n    ValueError:\n        if the capacity values are not integers, or the source or sink are out\n        of bounds.\n\n    Notes\n    -----\n    This solves the maximum flow problem on a given directed weighted graph:\n    A flow associates to every edge a value, also called a flow, less than the\n    capacity of the edge, so that for every vertex (apart from the source and\n    the sink vertices), the total incoming flow is equal to the total outgoing\n    flow. The value of a flow is the sum of the flow of all edges leaving the\n    source vertex, and the maximum flow problem consists of finding a flow\n    whose value is maximal.\n\n    By the max-flow min-cut theorem, the maximal value of the flow is also the\n    total weight of the edges in a minimum cut.\n\n    To solve the problem, we use the Edmonds--Karp algorithm. [1]_ This\n    particular implementation strives to exploit sparsity. Its time complexity\n    is :math:`O(VE^2)` and its space complexity is :math:`O(E)`.\n\n    The maximum flow problem is usually defined with real valued capacities,\n    but we require that all capacities are integral to ensure convergence. When\n    dealing with rational capacities, or capacities belonging to\n    :math:`x\\mathbb{Q}` for some fixed :math:`x \\in \\mathbb{R}`, it is possible\n    to reduce the problem to the integral case by scaling all capacities\n    accordingly.\n\n    Solving a maximum-flow problem can be used for example for graph cuts\n    optimization in computer vision [3]_.\n\n    References\n    ----------\n    .. [1] Edmonds, J. and Karp, R. M.\n           Theoretical improvements in algorithmic efficiency for network flow\n           problems. 1972. Journal of the ACM. 19 (2): pp. 248-264\n    .. [2] Cormen, T. H. and Leiserson, C. E. and Rivest, R. L. and Stein C.\n           Introduction to Algorithms. Second Edition. 2001. MIT Press.\n    .. [3] https://en.wikipedia.org/wiki/Graph_cuts_in_computer_vision\n\n    Examples\n    --------\n    Perhaps the simplest flow problem is that of a graph of only two vertices\n    with an edge from source (0) to sink (1)::\n\n        (0) --5--> (1)\n\n    Here, the maximum flow is simply the capacity of the edge:\n\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import maximum_flow\n    >>> graph = csr_matrix([[0, 5], [0, 0]])\n    >>> maximum_flow(graph, 0, 1).flow_value\n    5\n\n    If, on the other hand, there is a bottleneck between source and sink, that\n    can reduce the maximum flow::\n\n        (0) --5--> (1) --3--> (2)\n\n    >>> graph = csr_matrix([[0, 5, 0], [0, 0, 3], [0, 0, 0]])\n    >>> maximum_flow(graph, 0, 2).flow_value\n    3\n\n    A less trivial example is given in [2]_, Chapter 26.1:\n\n    >>> graph = csr_matrix([[0, 16, 13,  0,  0,  0],\n    ...                     [0, 10,  0, 12,  0,  0],\n    ...                     [0,  4,  0,  0, 14,  0],\n    ...                     [0,  0,  9,  0,  0, 20],\n    ...                     [0,  0,  0,  7,  0,  4],\n    ...                     [0,  0,  0,  0,  0,  0]])\n    >>> maximum_flow(graph, 0, 5).flow_value\n    23\n\n    It is possible to reduce the problem of finding a maximum matching in a\n    bipartite graph to a maximum flow problem: Let :math:`G = ((U, V), E)` be a\n    bipartite graph. Then, add to the graph a source vertex with edges to every\n    vertex in :math:`U` and a sink vertex with edges from every vertex in\n    :math:`V`. Finally, give every edge in the resulting graph a capacity of 1.\n    Then, a maximum flow in the new graph gives a maximum matching in the\n    original graph consisting of the edges in :math:`E` whose flow is positive.\n\n    Assume that the edges are represented by a\n    :math:`\\lvert U \\rvert \\times \\lvert V \\rvert` matrix in CSR format whose\n    :math:`(i, j)`'th entry is 1 if there is an edge from :math:`i \\in U` to\n    :math:`j \\in V` and 0 otherwise; that is, the input is of the form required\n    by :func:`maximum_bipartite_matching`. Then the CSR representation of the\n    graph constructed above contains this matrix as a block. Here's an example:\n\n    >>> graph = csr_matrix([[0, 1, 0, 1], [1, 0, 1, 0], [0, 1, 1, 0]])\n    >>> print(graph.toarray())\n    [[0 1 0 1]\n     [1 0 1 0]\n     [0 1 1 0]]\n    >>> i, j = graph.shape\n    >>> n = graph.nnz\n    >>> indptr = np.concatenate([[0],\n    ...                          graph.indptr + i,\n    ...                          np.arange(n + i + 1, n + i + j + 1),\n    ...                          [n + i + j]])\n    >>> indices = np.concatenate([np.arange(1, i + 1),\n    ...                           graph.indices + i + 1,\n    ...                           np.repeat(i + j + 1, j)])\n    >>> data = np.ones(n + i + j, dtype=int)\n    >>>\n    >>> graph_flow = csr_matrix((data, indices, indptr))\n    >>> print(graph_flow.toarray())\n    [[0 1 1 1 0 0 0 0 0]\n     [0 0 0 0 0 1 0 1 0]\n     [0 0 0 0 1 0 1 0 0]\n     [0 0 0 0 0 1 1 0 0]\n     [0 0 0 0 0 0 0 0 1]\n     [0 0 0 0 0 0 0 0 1]\n     [0 0 0 0 0 0 0 0 1]\n     [0 0 0 0 0 0 0 0 1]\n     [0 0 0 0 0 0 0 0 0]]\n\n    At this point, we can find the maximum flow between the added sink and the\n    added source and the desired matching can be obtained by restricting the\n    residual graph to the block corresponding to the original graph:\n\n    >>> flow = maximum_flow(graph_flow, 0, i+j+1)\n    >>> matching = flow.residual[1:i+1, i+1:i+j+1]\n    >>> print(matching.toarray())\n    [[0 1 0 0]\n     [1 0 0 0]\n     [0 0 1 0]]\n\n    This tells us that the first, second, and third vertex in :math:`U` are\n    matched with the second, first, and third vertex in :math:`V` respectively.\n\n    While this solves the maximum bipartite matching problem in general, note\n    that algorithms specialized to that problem, such as\n    :func:`maximum_bipartite_matching`, will generally perform better.\n\n    This approach can also be used to solve various common generalizations of\n    the maximum bipartite matching problem. If, for instance, some vertices can\n    be matched with more than one other vertex, this may be handled by\n    modifying the capacities of the new graph appropriately.\n\n    "
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, ClassVar, Sequence, List, TypeVar
from abc import ABCMeta, abstractmethod
from ..tree import (
    DecisionTreeRegressor as DecisionTreeRegressor,
    BaseDecisionTree as BaseDecisionTree,
    DecisionTreeClassifier as DecisionTreeClassifier,
)
from ..base import BaseEstimator
from joblib import effective_n_jobs as effective_n_jobs
from ..utils.metaestimators import _BaseComposition
from ..base import (
    clone as clone,
    is_classifier as is_classifier,
    is_regressor as is_regressor,
    MetaEstimatorMixin,
)
from .._typing import Int
from ..utils import Bunch, deprecated, check_random_state as check_random_state

_BaseHeterogeneousEnsemble_Self = TypeVar(
    "_BaseHeterogeneousEnsemble_Self", bound="_BaseHeterogeneousEnsemble"
)

import warnings

import numpy as np


class BaseEnsemble(MetaEstimatorMixin, BaseEstimator, metaclass=ABCMeta):
    estimators_: list[BaseEstimator] = ...

    # overwrite _required_parameters from MetaEstimatorMixin
    _required_parameters: ClassVar[List[str]] = ...

    @abstractmethod
    def __init__(
        self,
        estimator: Any = None,
        *,
        n_estimators: Int = 10,
        estimator_params: Sequence[str] = ...,
        base_estimator: Any = "deprecated",
    ) -> None:
        ...

    # TODO(1.4): remove
    # mypy error: Decorated property not supported
    @deprecated(  # type: ignore
        "Attribute `base_estimator_` was deprecated in version 1.2 and will be removed "
        "in 1.4. Use `estimator_` instead."
    )
    @property
    def base_estimator_(self) -> BaseEstimator:
        ...

    # TODO(1.4): remove
    @property
    def estimator_(self) -> BaseEstimator:
        ...

    def __len__(self) -> int:
        ...

    def __getitem__(self, index):
        ...

    def __iter__(self):
        ...


class _BaseHeterogeneousEnsemble(
    MetaEstimatorMixin, _BaseComposition, metaclass=ABCMeta
):
    estimators_: list[BaseEstimator] = ...

    _required_parameters: ClassVar[list] = ...

    @property
    def named_estimators(self) -> Bunch:
        ...

    @abstractmethod
    def __init__(self, estimators: list[tuple[str, BaseEstimator]]) -> None:
        ...

    def set_params(
        self: _BaseHeterogeneousEnsemble_Self, **params
    ) -> _BaseHeterogeneousEnsemble_Self:
        ...

    def get_params(self, deep: bool = True) -> dict:
        ...



from typing import Any
import math

from ..classes.graph import Graph
from ..utils import not_implemented_for, py_random_state

__all__ = ["spanner"]

@py_random_state(3)
def spanner(G: Graph, stretch: float, weight: Any = None, seed=None): ...



from .draw import (
    ellipse as ellipse,
    set_color as set_color,
    polygon_perimeter as polygon_perimeter,
    line as line,
    line_aa as line_aa,
    polygon as polygon,
    ellipse_perimeter as ellipse_perimeter,
    circle_perimeter as circle_perimeter,
    circle_perimeter_aa as circle_perimeter_aa,
    disk as disk,
    bezier_curve as bezier_curve,
    rectangle as rectangle,
    rectangle_perimeter as rectangle_perimeter,
)
from .draw3d import ellipsoid as ellipsoid, ellipsoid_stats as ellipsoid_stats
from ._random_shapes import random_shapes as random_shapes
from ._polygon2mask import polygon2mask as polygon2mask

from .draw_nd import line_nd as line_nd

__all__ = [
    "line",
    "line_aa",
    "line_nd",
    "bezier_curve",
    "polygon",
    "polygon_perimeter",
    "ellipse",
    "ellipse_perimeter",
    "ellipsoid",
    "ellipsoid_stats",
    "circle_perimeter",
    "circle_perimeter_aa",
    "disk",
    "set_color",
    "random_shapes",
    "rectangle",
    "rectangle_perimeter",
    "polygon2mask",
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2013, Nicolas P. Rougier. All rights reserved.
# Distributed under the terms of the new BSD License.
# -----------------------------------------------------------------------------

import numpy as np
from functools import reduce
from operator import mul

def dtype_reduce(dtype, level=0, depth=0): ...
def fetchcode(utype, prefix=""): ...



from decimal import Decimal
from itertools import chain
from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    Union,
)

from django.core.files.base import File
from django.forms.renderers import BaseRenderer
from django.utils.safestring import SafeText

_OptAttrs = Dict[str, Any]

class MediaOrderConflictWarning(RuntimeWarning): ...

class Media:
    _js: str
    def __init__(
        self,
        media: Optional[type] = ...,
        css: Optional[Dict[str, Iterable[str]]] = ...,
        js: Optional[Iterable[str]] = ...,
    ) -> None: ...
    def render(self) -> str: ...
    def render_js(self) -> List[str]: ...
    def render_css(self) -> chain[Any]: ...
    def absolute_path(self, path: str) -> str: ...
    def __getitem__(self, name: str) -> Media: ...
    @staticmethod
    def merge(list_1: Iterable[Any], list_2: Iterable[Any]) -> Iterable[Any]: ...
    def __add__(self, other: Media) -> Media: ...

class MediaDefiningClass(type): ...

class Widget:
    needs_multipart_form: bool = ...
    is_localized: bool = ...
    is_required: bool = ...
    supports_microseconds: bool = ...
    attrs: _OptAttrs = ...
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...
    @property
    def is_hidden(self) -> bool: ...
    def subwidgets(
        self, name: str, value: Optional[List[str]], attrs: _OptAttrs = ...
    ) -> Iterator[Dict[str, Any]]: ...
    def format_value(self, value: Any) -> Optional[str]: ...
    def get_context(
        self, name: str, value: Any, attrs: Optional[_OptAttrs]
    ) -> Dict[str, Any]: ...
    def render(
        self,
        name: str,
        value: Any,
        attrs: Optional[_OptAttrs] = ...,
        renderer: Optional[BaseRenderer] = ...,
    ) -> SafeText: ...
    def build_attrs(
        self, base_attrs: _OptAttrs, extra_attrs: Optional[_OptAttrs] = ...
    ) -> Dict[str, Union[Decimal, float, str]]: ...
    def value_from_datadict(
        self, data: Dict[str, Any], files: Mapping[str, Iterable[Any]], name: str
    ) -> Any: ...
    def value_omitted_from_data(
        self, data: Dict[str, Any], files: Mapping[str, Iterable[Any]], name: str
    ) -> bool: ...
    def id_for_label(self, id_: str) -> str: ...
    def use_required_attribute(self, initial: Any) -> bool: ...

class Input(Widget):
    input_type: str = ...
    template_name: str = ...

class TextInput(Input): ...
class NumberInput(Input): ...
class EmailInput(Input): ...
class URLInput(Input): ...

class PasswordInput(Input):
    render_value: bool = ...
    def __init__(
        self, attrs: Optional[_OptAttrs] = ..., render_value: bool = ...
    ) -> None: ...

class HiddenInput(Input):
    choices: Iterable[Tuple[str, str]]

class MultipleHiddenInput(HiddenInput): ...
class FileInput(Input): ...

FILE_INPUT_CONTRADICTION: Any

class ClearableFileInput(FileInput):
    clear_checkbox_label: Any = ...
    initial_text: Any = ...
    input_text: Any = ...
    def clear_checkbox_name(self, name: str) -> str: ...
    def clear_checkbox_id(self, name: str) -> str: ...
    def is_initial(self, value: Optional[Union[File, str]]) -> bool: ...

class Textarea(Widget):
    template_name: str = ...

class DateTimeBaseInput(TextInput):
    format_key: str = ...
    format: Optional[str] = ...
    def __init__(
        self, attrs: Optional[_OptAttrs] = ..., format: Optional[str] = ...
    ) -> None: ...

class DateInput(DateTimeBaseInput): ...
class DateTimeInput(DateTimeBaseInput): ...
class TimeInput(DateTimeBaseInput): ...

class CheckboxInput(Input):
    check_test: Callable[..., Any] = ...
    def __init__(
        self,
        attrs: Optional[_OptAttrs] = ...,
        check_test: Optional[Callable[..., Any]] = ...,
    ) -> None: ...

class ChoiceWidget(Widget):
    allow_multiple_selected: bool = ...
    input_type: Optional[str] = ...
    template_name: Optional[str] = ...
    option_template_name: str = ...
    add_id_index: bool = ...
    checked_attribute: Any = ...
    option_inherits_attrs: bool = ...
    choices: List[List[Union[int, str]]] = ...
    def __init__(
        self, attrs: Optional[_OptAttrs] = ..., choices: Sequence[Tuple[Any, Any]] = ...
    ) -> None: ...
    def options(
        self, name: str, value: List[str], attrs: Optional[_OptAttrs] = ...
    ) -> None: ...
    def optgroups(
        self, name: str, value: List[str], attrs: Optional[_OptAttrs] = ...
    ) -> Any: ...
    def create_option(
        self,
        name: str,
        value: Any,
        label: Union[int, str],
        selected: Union[Set[str], bool],
        index: int,
        subindex: Optional[int] = ...,
        attrs: Optional[_OptAttrs] = ...,
    ) -> Dict[str, Any]: ...
    def id_for_label(self, id_: str, index: str = ...) -> str: ...

class Select(ChoiceWidget): ...
class NullBooleanSelect(Select): ...

class SelectMultiple(Select):
    allow_multiple_selected: bool = ...

class RadioSelect(ChoiceWidget):
    can_add_related: bool

class CheckboxSelectMultiple(ChoiceWidget): ...

class MultiWidget(Widget):
    template_name: str = ...
    widgets: List[Widget] = ...
    def __init__(
        self,
        widgets: Sequence[Union[Widget, Type[Widget]]],
        attrs: Optional[_OptAttrs] = ...,
    ) -> None: ...
    def decompress(self, value: Any) -> Optional[Any]: ...
    media: Any = ...

class SplitDateTimeWidget(MultiWidget):
    def __init__(
        self,
        attrs: Optional[_OptAttrs] = ...,
        date_format: Optional[str] = ...,
        time_format: Optional[str] = ...,
        date_attrs: Optional[Dict[str, str]] = ...,
        time_attrs: Optional[Dict[str, str]] = ...,
    ) -> None: ...

class SplitHiddenDateTimeWidget(SplitDateTimeWidget): ...

class SelectDateWidget(Widget):
    none_value: Any = ...
    month_field: str = ...
    day_field: str = ...
    year_field: str = ...
    template_name: str = ...
    input_type: str = ...
    select_widget: Any = ...
    date_re: Any = ...
    years: Any = ...
    months: Any = ...
    year_none_value: Any = ...
    month_none_value: Any = ...
    day_none_value: Any = ...
    def __init__(
        self,
        attrs: Optional[_OptAttrs] = ...,
        years: Optional[Iterable[Union[int, str]]] = ...,
        months: Optional[Dict[int, str]] = ...,
        empty_label: Optional[Union[str, Sequence[str]]] = ...,
    ) -> None: ...



import decimal
import io
import numpy.typing
import pandas as pd

Decimal = decimal.Decimal
PythonScalar = str | int | float | bool

ArrayLike = numpy.typing.ArrayLike
FileLike = io.IOBase
PathLike = str

PandasScalar = pd.Period | pd.Timestamp | pd.Timedelta | pd.Interval
Scalar = PythonScalar | PandasScalar

Color = tuple[float, float, float] | str

__all__ = [
    "ArrayLike",
    "Color",
    "Decimal",
    "FileLike",
    "PathLike",
    "Scalar",
]



from typing import Any, ClassVar, Literal, TypeVar
from ..utils.sparsefuncs import (
    inplace_column_scale as inplace_column_scale,
    mean_variance_axis as mean_variance_axis,
    incr_mean_variance_axis as incr_mean_variance_axis,
    min_max_axis as min_max_axis,
)
from numpy.random import RandomState
from scipy.special import boxcox as boxcox
from scipy import sparse as sparse, stats as stats, optimize as optimize
from ..utils.sparsefuncs_fast import (
    inplace_csr_row_normalize_l1 as inplace_csr_row_normalize_l1,
    inplace_csr_row_normalize_l2 as inplace_csr_row_normalize_l2,
)
from ._encoders import OneHotEncoder
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ..utils.extmath import row_norms as row_norms
from numbers import Integral as Integral, Real as Real
from pandas.core.series import Series
from ..base import (
    BaseEstimator,
    TransformerMixin,
    OneToOneFeatureMixin,
    ClassNamePrefixFeaturesOutMixin,
)
from scipy.sparse import spmatrix
from scipy.sparse._csr import csr_matrix
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import check_array as check_array
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_random_state as check_random_state,
    FLOAT_DTYPES as FLOAT_DTYPES,
)

RobustScaler_Self = TypeVar("RobustScaler_Self", bound="RobustScaler")
KernelCenterer_Self = TypeVar("KernelCenterer_Self", bound="KernelCenterer")
Binarizer_Self = TypeVar("Binarizer_Self", bound="Binarizer")
QuantileTransformer_Self = TypeVar(
    "QuantileTransformer_Self", bound="QuantileTransformer"
)
MaxAbsScaler_Self = TypeVar("MaxAbsScaler_Self", bound="MaxAbsScaler")
StandardScaler_Self = TypeVar("StandardScaler_Self", bound="StandardScaler")
MinMaxScaler_Self = TypeVar("MinMaxScaler_Self", bound="MinMaxScaler")
Normalizer_Self = TypeVar("Normalizer_Self", bound="Normalizer")
PowerTransformer_Self = TypeVar("PowerTransformer_Self", bound="PowerTransformer")

# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Mathieu Blondel <mathieu@mblondel.org>
#          Olivier Grisel <olivier.grisel@ensta.org>
#          Andreas Mueller <amueller@ais.uni-bonn.de>
#          Eric Martin <eric@ericmart.in>
#          Giorgio Patrini <giorgio.patrini@anu.edu.au>
#          Eric Chang <ericchang2017@u.northwestern.edu>
# License: BSD 3 clause


import warnings

import numpy as np


BOUNDS_THRESHOLD: float = ...

__all__ = [
    "Binarizer",
    "KernelCenterer",
    "MinMaxScaler",
    "MaxAbsScaler",
    "Normalizer",
    "OneHotEncoder",
    "RobustScaler",
    "StandardScaler",
    "QuantileTransformer",
    "PowerTransformer",
    "add_dummy_feature",
    "binarize",
    "normalize",
    "scale",
    "robust_scale",
    "maxabs_scale",
    "minmax_scale",
    "quantile_transform",
    "power_transform",
]


def scale(
    X: MatrixLike | ArrayLike,
    *,
    axis: Int = 0,
    with_mean: bool = True,
    with_std: bool = True,
    copy: bool = True,
) -> ndarray | spmatrix:
    ...


class MinMaxScaler(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_samples_seen_: int = ...
    n_features_in_: int = ...
    data_range_: ndarray = ...
    data_max_: ndarray = ...
    data_min_: ndarray = ...
    scale_: ndarray = ...
    min_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        feature_range: tuple[int, int] = ...,
        *,
        copy: bool = True,
        clip: bool = False,
    ) -> None:
        ...

    def fit(
        self: MinMaxScaler_Self, X: MatrixLike, y: Series | None | ndarray = None
    ) -> MinMaxScaler_Self:
        ...

    def partial_fit(
        self: MinMaxScaler_Self, X: MatrixLike, y: Series | None | ndarray = None
    ) -> MinMaxScaler_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...


def minmax_scale(
    X: MatrixLike,
    feature_range: tuple[int, int] = ...,
    *,
    axis: Int = 0,
    copy: bool = True,
) -> ndarray:
    ...


class StandardScaler(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    n_samples_seen_: ndarray | int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    var_: None | ndarray = ...
    mean_: None | ndarray = ...
    scale_: None | ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, *, copy: bool = True, with_mean: bool = True, with_std: bool = True
    ) -> None:
        ...

    def fit(
        self: StandardScaler_Self,
        X: MatrixLike | ArrayLike,
        y: Series | None | ndarray | list[int] = None,
        sample_weight: None | ArrayLike = None,
    ) -> StandardScaler_Self:
        ...

    def partial_fit(
        self: StandardScaler_Self,
        X: MatrixLike | ArrayLike,
        y: Series | None | ndarray | list[int] = None,
        sample_weight: None | ArrayLike = None,
    ) -> StandardScaler_Self:
        ...

    def transform(self, X: MatrixLike, copy: None | bool = None) -> ndarray | spmatrix:
        ...

    def inverse_transform(
        self, X: MatrixLike | ArrayLike, copy: None | bool = None
    ) -> ndarray | spmatrix:
        ...


class MaxAbsScaler(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    n_samples_seen_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    max_abs_: ndarray = ...
    scale_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, *, copy: bool = True) -> None:
        ...

    def fit(
        self: MaxAbsScaler_Self, X: MatrixLike | ArrayLike, y=None
    ) -> MaxAbsScaler_Self:
        ...

    def partial_fit(
        self: MaxAbsScaler_Self, X: MatrixLike | ArrayLike, y=None
    ) -> MaxAbsScaler_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...


def maxabs_scale(X: MatrixLike | ArrayLike, *, axis: Int = 0, copy: bool = True):
    ...


class RobustScaler(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    scale_: ndarray = ...
    center_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        with_centering: bool = True,
        with_scaling: bool = True,
        quantile_range: tuple[float, float] = ...,
        copy: bool = True,
        unit_variance: bool = False,
    ) -> None:
        ...

    def fit(
        self: RobustScaler_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> RobustScaler_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...


def robust_scale(
    X: MatrixLike,
    *,
    axis: Int = 0,
    with_centering: bool = True,
    with_scaling: bool = True,
    quantile_range: tuple[float, float] = ...,
    copy: bool = True,
    unit_variance: bool = False,
) -> ndarray | spmatrix:
    ...


def normalize(
    X: MatrixLike | ArrayLike,
    norm: Literal["l1", "l2", "max", "l2"] = "l2",
    *,
    axis: int = 1,
    copy: bool = True,
    return_norm: bool = False,
) -> csr_matrix | tuple[ndarray | spmatrix, ndarray] | ndarray:
    ...


class Normalizer(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, norm: Literal["l1", "l2", "max", "l2"] = "l2", *, copy: bool = True
    ) -> None:
        ...

    def fit(
        self: Normalizer_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> Normalizer_Self:
        ...

    def transform(
        self, X: MatrixLike | ArrayLike, copy: None | bool = None
    ) -> ndarray | spmatrix:
        ...


def binarize(
    X: MatrixLike | ArrayLike, *, threshold: Float = 0.0, copy: bool = True
) -> ndarray | spmatrix:
    ...


class Binarizer(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, *, threshold: Float = 0.0, copy: bool = True) -> None:
        ...

    def fit(self: Binarizer_Self, X: MatrixLike | ArrayLike, y=None) -> Binarizer_Self:
        ...

    def transform(
        self, X: MatrixLike | ArrayLike, copy: None | bool = None
    ) -> ndarray | spmatrix:
        ...


class KernelCenterer(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    K_fit_all_: float = ...
    K_fit_rows_: ndarray = ...

    def __init__(self) -> None:
        ...

    def fit(self: KernelCenterer_Self, K: MatrixLike, y=None) -> KernelCenterer_Self:
        ...

    def transform(self, K: MatrixLike, copy: bool = True) -> ndarray:
        ...


def add_dummy_feature(
    X: MatrixLike | ArrayLike, value: Float = 1.0
) -> ndarray | spmatrix:
    ...


class QuantileTransformer(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    references_: ndarray = ...
    quantiles_: ndarray = ...
    n_quantiles_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_quantiles: Int = 1000,
        output_distribution: Literal["uniform", "normal", "uniform"] = "uniform",
        ignore_implicit_zeros: bool = False,
        subsample: Int = 10_000,
        random_state: RandomState | None | Int = None,
        copy: bool = True,
    ) -> None:
        ...

    def fit(
        self: QuantileTransformer_Self,
        X: MatrixLike | ArrayLike,
        y: Series | None = None,
    ) -> QuantileTransformer_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...


def quantile_transform(
    X: MatrixLike | ArrayLike,
    *,
    axis: Int = 0,
    n_quantiles: Int = 1000,
    output_distribution: Literal["uniform", "normal", "uniform"] = "uniform",
    ignore_implicit_zeros: bool = False,
    subsample: Int = ...,
    random_state: RandomState | None | Int = None,
    copy: bool = True,
) -> ndarray | spmatrix:
    ...


class PowerTransformer(OneToOneFeatureMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    lambdas_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        method: Literal["yeo-johnson", "box-cox", "yeo-johnson"] = "yeo-johnson",
        *,
        standardize: bool = True,
        copy: bool = True,
    ) -> None:
        ...

    def fit(
        self: PowerTransformer_Self, X: MatrixLike, y=None
    ) -> PowerTransformer_Self:
        ...

    def fit_transform(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...


def power_transform(
    X: MatrixLike,
    method: Literal["yeo-johnson", "box-cox", "yeo-johnson"] = "yeo-johnson",
    *,
    standardize: bool = True,
    copy: bool = True,
) -> ndarray:
    ...



from math import log

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "resource_allocation_index",
    "jaccard_coefficient",
    "adamic_adar_index",
    "preferential_attachment",
    "cn_soundarajan_hopcroft",
    "ra_index_soundarajan_hopcroft",
    "within_inter_cluster",
    "common_neighbor_centrality",
]

def resource_allocation_index(G: Graph, ebunch=None): ...
def jaccard_coefficient(G: Graph, ebunch=None): ...
def adamic_adar_index(G: Graph, ebunch=None): ...
def common_neighbor_centrality(G: Graph, ebunch=None, alpha=0.8): ...
def preferential_attachment(G: Graph, ebunch=None): ...
def cn_soundarajan_hopcroft(G: Graph, ebunch=None, community="community"): ...
def ra_index_soundarajan_hopcroft(G: Graph, ebunch=None, community="community"): ...
def within_inter_cluster(G: Graph, ebunch=None, delta=0.001, community="community"): ...



from numpy.typing import ArrayLike
import math
import numbers
from functools import reduce
from ...classes.graph import Graph

from ...utils import nodes_or_number, py_random_state

__all__ = [
    "configuration_model",
    "havel_hakimi_graph",
    "reverse_havel_hakimi_graph",
    "alternating_havel_hakimi_graph",
    "preferential_attachment_graph",
    "random_graph",
    "gnmk_random_graph",
    "complete_bipartite_graph",
]

def complete_bipartite_graph(n1, n2, create_using=None): ...
@py_random_state(3)
def configuration_model(
    aseq: ArrayLike, bseq: ArrayLike, create_using=None, seed=None
): ...
def havel_hakimi_graph(aseq: ArrayLike, bseq: ArrayLike, create_using=None): ...
def reverse_havel_hakimi_graph(aseq: ArrayLike, bseq: ArrayLike, create_using=None): ...
def alternating_havel_hakimi_graph(
    aseq: ArrayLike, bseq: ArrayLike, create_using=None
): ...
@py_random_state(3)
def preferential_attachment_graph(
    aseq: ArrayLike, p: float, create_using=None, seed=None
): ...
@py_random_state(3)
def random_graph(n: int, m: int, p: float, seed=None, directed=False): ...
@py_random_state(3)
def gnmk_random_graph(n: int, m: int, k: int, seed=None, directed=False): ...




import numpy as np
from scipy.sparse import spmatrix


def init_bounds_dense(
        X: np.ndarray,
        centers: np.ndarray,
        center_half_distances: np.ndarray,
        labels: np.ndarray,
        upper_bounds: np.ndarray,
        lower_bounds: np.ndarray,
        n_threads: int) -> None:
    """Initialize upper and lower bounds for each sample for dense input data.
    Given X, centers and the pairwise distances divided by 2.0 between the
    centers this calculates the upper bounds and lower bounds for each sample.
    The upper bound for each sample is set to the distance between the sample
    and the closest center.
    The lower bound for each sample is a one-dimensional array of n_clusters.
    For each sample i assume that the previously assigned cluster is c1 and the
    previous closest distance is dist, for a new cluster c2, the
    lower_bound[i][c2] is set to distance between the sample and this new
    cluster, if and only if dist > center_half_distances[c1][c2]. This prevents
    computation of unnecessary distances for each sample to the clusters that
    it is unlikely to be assigned to.
    Parameters
    ----------
    X : ndarray of shape (n_samples, n_features), dtype=floating
        The input data.
    centers : ndarray of shape (n_clusters, n_features), dtype=floating
        The cluster centers.
    center_half_distances : ndarray of shape (n_clusters, n_clusters),             dtype=floating
        The half of the distance between any 2 clusters centers.
    labels : ndarray of shape(n_samples), dtype=int
        The label for each sample. This array is modified in place.
    upper_bounds : ndarray of shape(n_samples,), dtype=floating
        The upper bound on the distance between each sample and its closest
        cluster center. This array is modified in place.
    lower_bounds : ndarray, of shape(n_samples, n_clusters), dtype=floating
        The lower bound on the distance between each sample and each cluster
        center. This array is modified in place.
    n_threads : int
        The number of threads to be used by openmp.
    """
    ...

def init_bounds_sparse(
        X: spmatrix,
        centers: np.ndarray,
        center_half_distances: np.ndarray,
        labels: np.ndarray,
        upper_bounds: np.ndarray,
        lower_bounds: np.ndarray,
        n_threads: int) -> None:
    """Initialize upper and lower bounds for each sample for sparse input data.
    Given X, centers and the pairwise distances divided by 2.0 between the
    centers this calculates the upper bounds and lower bounds for each sample.
    The upper bound for each sample is set to the distance between the sample
    and the closest center.
    The lower bound for each sample is a one-dimensional array of n_clusters.
    For each sample i assume that the previously assigned cluster is c1 and the
    previous closest distance is dist, for a new cluster c2, the
    lower_bound[i][c2] is set to distance between the sample and this new
    cluster, if and only if dist > center_half_distances[c1][c2]. This prevents
    computation of unnecessary distances for each sample to the clusters that
    it is unlikely to be assigned to.
    Parameters
    ----------
    X : sparse matrix of shape (n_samples, n_features), dtype=floating
        The input data. Must be in CSR format.
    centers : ndarray of shape (n_clusters, n_features), dtype=floating
        The cluster centers.
    center_half_distances : ndarray of shape (n_clusters, n_clusters),             dtype=floating
        The half of the distance between any 2 clusters centers.
    labels : ndarray of shape(n_samples), dtype=int
        The label for each sample. This array is modified in place.
    upper_bounds : ndarray of shape(n_samples,), dtype=floating
        The upper bound on the distance between each sample and its closest
        cluster center. This array is modified in place.
    lower_bounds : ndarray of shape(n_samples, n_clusters), dtype=floating
        The lower bound on the distance between each sample and each cluster
        center. This array is modified in place.
    n_threads : int
        The number of threads to be used by openmp.
    """
    ...


def elkan_iter_chunked_dense(
        X: np.ndarray,
        sample_weight: np.ndarray,
        centers_old: np.ndarray,
        centers_new: np.ndarray,
        weight_in_clusters: np.ndarray,
        center_half_distances: np.ndarray,
        distance_next_center: np.ndarray,
        upper_bounds: np.ndarray,
        lower_bounds: np.ndarray,
        labels: np.ndarray,
        center_shift: np.ndarray,
        n_threads: int,
        update_centers: bool=True) -> None:
    """Single iteration of K-means Elkan algorithm with dense input.
    Update labels and centers (inplace), for one iteration, distributed
    over data chunks.
    Parameters
    ----------
    X : ndarray of shape (n_samples, n_features), dtype=floating
        The observations to cluster.
    sample_weight : ndarray of shape (n_samples,), dtype=floating
        The weights for each observation in X.
    centers_old : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers before previous iteration, placeholder for the centers after
        previous iteration.
    centers_new : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers after previous iteration, placeholder for the new centers
        computed during this iteration.
    weight_in_clusters : ndarray of shape (n_clusters,), dtype=floating
        Placeholder for the sums of the weights of every observation assigned
        to each center.
    center_half_distances : ndarray of shape (n_clusters, n_clusters),             dtype=floating
        Half pairwise distances between centers.
    distance_next_center : ndarray of shape (n_clusters,), dtype=floating
        Distance between each center its closest center.
    upper_bounds : ndarray of shape (n_samples,), dtype=floating
        Upper bound for the distance between each sample and its center,
        updated inplace.
    lower_bounds : ndarray of shape (n_samples, n_clusters), dtype=floating
        Lower bound for the distance between each sample and each center,
        updated inplace.
    labels : ndarray of shape (n_samples,), dtype=int
        labels assignment.
    center_shift : ndarray of shape (n_clusters,), dtype=floating
        Distance between old and new centers.
    n_threads : int
        The number of threads to be used by openmp.
    update_centers : bool
        - If True, the labels and the new centers will be computed, i.e. runs
          the E-step and the M-step of the algorithm.
        - If False, only the labels will be computed, i.e runs the E-step of
          the algorithm. This is useful especially when calling predict on a
          fitted model.
    """
    ...


def elkan_iter_chunked_sparse(
        X: spmatrix,
        sample_weight: np.ndarray,
        centers_old: np.ndarray,
        centers_new: np.ndarray,
        weight_in_clusters: np.ndarray,
        center_half_distances: np.ndarray,
        distance_next_center: np.ndarray,
        upper_bounds: np.ndarray,
        lower_bounds: np.ndarray,
        labels: np.ndarray,
        center_shift: np.ndarray,
        n_threads: int,
        update_centers: bool=True) -> None:
    """Single iteration of K-means Elkan algorithm with sparse input.
    Update labels and centers (inplace), for one iteration, distributed
    over data chunks.
    Parameters
    ----------
    X : sparse matrix of shape (n_samples, n_features)
        The observations to cluster. Must be in CSR format.
    sample_weight : ndarray of shape (n_samples,), dtype=floating
        The weights for each observation in X.
    centers_old : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers before previous iteration, placeholder for the centers after
        previous iteration.
    centers_new : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers after previous iteration, placeholder for the new centers
        computed during this iteration.
    weight_in_clusters : ndarray of shape (n_clusters,), dtype=floating
        Placeholder for the sums of the weights of every observation assigned
        to each center.
    center_half_distances : ndarray of shape (n_clusters, n_clusters),             dtype=floating
        Half pairwise distances between centers.
    distance_next_center : ndarray of shape (n_clusters,), dtype=floating
        Distance between each center its closest center.
    upper_bounds : ndarray of shape (n_samples,), dtype=floating
        Upper bound for the distance between each sample and its center,
        updated inplace.
    lower_bounds : ndarray of shape (n_samples, n_clusters), dtype=floating
        Lower bound for the distance between each sample and each center,
        updated inplace.
    labels : ndarray of shape (n_samples,), dtype=int
        labels assignment.
    center_shift : ndarray of shape (n_clusters,), dtype=floating
        Distance between old and new centers.
    n_threads : int
        The number of threads to be used by openmp.
    update_centers : bool
        - If True, the labels and the new centers will be computed, i.e. runs
          the E-step and the M-step of the algorithm.
        - If False, only the labels will be computed, i.e runs the E-step of
          the algorithm. This is useful especially when calling predict on a
          fitted model.
    """
    ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.signal.spline, version: 0.2
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__version__: str
def cspline2d(input, lambda_=..., precision=...) -> typing.Any:
    'out = cspline2d(input, lambda=0.0, precision=-1.0)\n\n    Coefficients for 2-D cubic (3rd order) B-spline.\n\n    Return the third-order B-spline coefficients over a regularly spaced\n    input grid for the two-dimensional input image.\n\n    Parameters\n    ----------\n    input : ndarray\n        The input signal.\n    lambda : float\n        Specifies the amount of smoothing in the transfer function.\n    precision : float\n        Specifies the precision for computing the infinite sum needed to apply mirror-\n        symmetric boundary conditions.\n\n    Returns\n    -------\n    output : ndarray\n        The filtered signal.\n\n    Examples\n    --------\n    Examples are given :ref:`in the tutorial <tutorial-signal-bsplines>`.\n\n'
    ...

def qspline2d(input, lambda_=..., precision=...) -> typing.Any:
    'out = qspline2d(input, lambda=0.0, precision=-1.0)\n\n    Coefficients for 2-D quadratic (2nd order) B-spline:\n\n    Return the second-order B-spline coefficients over a regularly spaced\n    input grid for the two-dimensional input image.\n\n    Parameters\n    ----------\n    input : ndarray\n        The input signal.\n    lambda : float\n        Specifies the amount of smoothing in the transfer function.\n    precision : float\n        Specifies the precision for computing the infinite sum needed to apply mirror-\n        symmetric boundary conditions.\n\n    Returns\n    -------\n    output : ndarray\n        The filtered signal.\n\n    Examples\n    --------\n    Examples are given :ref:`in the tutorial <tutorial-signal-bsplines>`.\n\n'
    ...

def sepfir2d(input, hrow, hcol) -> typing.Any:
    'out = sepfir2d(input, hrow, hcol)\n\n    Convolve with a 2-D separable FIR filter.\n\n    Convolve the rank-2 input array with the separable filter defined by the\n    rank-1 arrays hrow, and hcol. Mirror symmetric boundary conditions are\n    assumed. This function can be used to find an image given its B-spline\n    representation.\n\n    Parameters\n    ----------\n    input : ndarray\n        The input signal. Must be a rank-2 array.\n    hrow : ndarray\n        A rank-1 array defining the row direction of the filter.\n        Must be odd-length\n    hcol : ndarray\n        A rank-1 array defining the column direction of the filter.\n        Must be odd-length\n\n    Returns\n    -------\n    output : ndarray\n        The filtered signal.\n\n    Examples\n    --------\n    Examples are given :ref:`in the tutorial <tutorial-signal-bsplines>`.\n\n'
    ...

def symiirorder1(input, c0, z1, precision=...) -> typing.Any:
    'out = symiirorder1(input, c0, z1, precision=-1.0)\n\n    Implement a smoothing IIR filter with mirror-symmetric boundary conditions\n    using a cascade of first-order sections.  The second section uses a\n    reversed sequence.  This implements a system with the following\n    transfer function and mirror-symmetric boundary conditions::\n\n                           c0              \n           H(z) = ---------------------    \n                   (1-z1/z) (1 - z1 z)     \n\n    The resulting signal will have mirror symmetric boundary conditions as well.\n\n    Parameters\n    ----------\n    input : ndarray\n        The input signal.\n    c0, z1 : scalar\n        Parameters in the transfer function.\n    precision :\n        Specifies the precision for calculating initial conditions\n        of the recursive filter based on mirror-symmetric input.\n\n    Returns\n    -------\n    output : ndarray\n        The filtered signal.'
    ...

def symiirorder2(input, r, omega, precision=...) -> typing.Any:
    'out = symiirorder2(input, r, omega, precision=-1.0)\n\n    Implement a smoothing IIR filter with mirror-symmetric boundary conditions\n    using a cascade of second-order sections.  The second section uses a\n    reversed sequence.  This implements the following transfer function::\n\n                                  cs^2\n         H(z) = ---------------------------------------\n                (1 - a2/z - a3/z^2) (1 - a2 z - a3 z^2 )\n\n    where::\n\n          a2 = (2 r cos omega)\n          a3 = - r^2\n          cs = 1 - 2 r cos omega + r^2\n\n    Parameters\n    ----------\n    input : ndarray\n        The input signal.\n    r, omega : float\n        Parameters in the transfer function.\n    precision : float\n        Specifies the precision for calculating initial conditions\n        of the recursive filter based on mirror-symmetric input.\n\n    Returns\n    -------\n    output : ndarray\n        The filtered signal.'
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Literal, Sequence
from ._testing import ignore_warnings as ignore_warnings
from operator import itemgetter as itemgetter
from numpy import ndarray
from ..base import (
    BaseEstimator as BaseEstimator,
    ClassifierMixin as ClassifierMixin,
    RegressorMixin as RegressorMixin,
    TransformerMixin as TransformerMixin,
    ClusterMixin as ClusterMixin,
)
from pathlib import Path as Path
from importlib import import_module as import_module
from . import IS_PYPY as IS_PYPY
import pkgutil
import inspect

_MODULE_TO_IGNORE: set = ...


def all_estimators(
    type_filter: None
    | Sequence[Literal["classifier", "regressor", "cluster", "transformer"]]
    | Literal["classifier", "regressor", "cluster", "transformer"] = None
) -> ndarray:
    ...


def all_displays() -> ndarray:
    ...


def all_functions() -> ndarray:
    ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .application import Application

# Initialize default app
# Only for use within *this* module.
# One should always call use_app() to obtain the default app.
default_app: None = ...

def use_app(backend_name: str | None = None, call_reuse: bool = True): ...
def create(): ...
def run(): ...
def quit(): ...
def process_events(): ...



"""
This type stub file was generated by pyright.
"""

"""Windows-specific implementation of process utilities.

This file is only meant to be imported by process.py, not by end-users.
"""
class AvoidUNCPath:
    """A context manager to protect command execution from UNC paths.

    In the Win32 API, commands can't be invoked with the cwd being a UNC path.
    This context manager temporarily changes directory to the 'C:' drive on
    entering, and restores the original working directory on exit.

    The context manager returns the starting working directory *if* it made a
    change and None otherwise, so that users can apply the necessary adjustment
    to their system calls in the event of a change.

    Examples
    --------
    ::
        cmd = 'dir'
        with AvoidUNCPath() as path:
            if path is not None:
                cmd = '"pushd %s &&"%s' % (path, cmd)
            os.system(cmd)
    """
    def __enter__(self): # -> None:
        ...
    
    def __exit__(self, exc_type, exc_value, traceback): # -> None:
        ...
    


def system(cmd): # -> None:
    """Win32 version of os.system() that works with network shares.

    Note that this implementation returns None, as meant for use in IPython.

    Parameters
    ----------
    cmd : str or list
        A command to be executed in the system shell.

    Returns
    -------
    int : child process' exit code.
    """
    ...

def getoutput(cmd):
    """Return standard output of executing cmd in a shell.

    Accepts the same arguments as os.system().

    Parameters
    ----------
    cmd : str or list
        A command to be executed in the system shell.

    Returns
    -------
    stdout : str
    """
    ...

CommandLineToArgvW = ...
LocalFree = ...
def arg_split(commandline, posix=..., strict=...):
    """Split a command line's arguments in a shell-like manner.

        This is a special version for windows that use a ctypes call to CommandLineToArgvW
        to do the argv splitting. The posix parameter is ignored.

        If strict=False, process_common.arg_split(...strict=False) is used instead.
        """
    ...

def check_pid(pid):
    ...




from typing import Callable, Iterable, Sequence
from ._typing import *
from .transforms import Transform
from .backend_bases import GraphicsContextBase, RendererBase

class AbstractPathEffect:
    def __init__(self, offset: Sequence[float] = ...) -> None: ...
    def draw_path(
        self,
        renderer: RendererBase,
        gc: GraphicsContextBase,
        tpath,
        affine,
        rgbFace=...,
    ): ...

class PathEffectRenderer(RendererBase):
    def __init__(
        self, path_effects: Iterable[AbstractPathEffect], renderer: RendererBase
    ) -> None: ...
    def copy_with_path_effect(self, path_effects): ...
    def draw_path(self, gc: GraphicsContextBase, tpath, affine, rgbFace=...): ...
    def draw_markers(
        self,
        gc: GraphicsContextBase,
        marker_path,
        marker_trans: Transform,
        path,
        *args,
        **kwargs
    ): ...
    def draw_path_collection(
        self, gc: GraphicsContextBase, master_transform, paths, *args, **kwargs
    ): ...
    def __getattribute__(
        self, name: str
    ) -> Callable | list[Stroke | Normal] | RendererBase: ...

class Normal(AbstractPathEffect): ...

class Stroke(AbstractPathEffect):
    def __init__(self, offset: Sequence[float] = ..., **kwargs) -> None: ...
    def draw_path(
        self,
        renderer: RendererBase,
        gc: GraphicsContextBase,
        tpath,
        affine,
        rgbFace: Color,
    ): ...

withStroke = ...

class SimplePatchShadow(AbstractPathEffect):
    def __init__(
        self,
        offset: Sequence[float] = ...,
        shadow_rgbFace: Color = ...,
        alpha: float = 0.3,
        rho: float = 0.3,
        **kwargs
    ) -> None: ...
    def draw_path(
        self,
        renderer: RendererBase,
        gc: GraphicsContextBase,
        tpath,
        affine,
        rgbFace: Color,
    ): ...

withSimplePatchShadow = ...

class SimpleLineShadow(AbstractPathEffect):
    def __init__(
        self,
        offset: Sequence[float] = ...,
        shadow_color: Color = "black",
        alpha: float = 0.3,
        rho: float = 0.3,
        **kwargs
    ) -> None: ...
    def draw_path(
        self,
        renderer: RendererBase,
        gc: GraphicsContextBase,
        tpath,
        affine,
        rgbFace: Color,
    ): ...

class PathPatchEffect(AbstractPathEffect):
    def __init__(self, offset: Sequence[float] = ..., **kwargs) -> None: ...
    def draw_path(
        self,
        renderer: RendererBase,
        gc: GraphicsContextBase,
        tpath,
        affine,
        rgbFace: Color,
    ): ...

class TickedStroke(AbstractPathEffect):
    def __init__(
        self,
        offset: Sequence[float] = ...,
        spacing: float = 10,
        angle: float = 45,
        length: float = 1.414,
        **kwargs
    ) -> None: ...
    def draw_path(
        self,
        renderer: RendererBase,
        gc: GraphicsContextBase,
        tpath,
        affine,
        rgbFace: Color,
    ): ...

withTickedStroke = ...



from typing import Any, Optional

from django.db.models.expressions import Func

class Aggregate(Func):
    filter_template: str = ...
    filter: Any = ...
    allow_distinct: bool = ...
    def __init__(
        self,
        *expressions: Any,
        distinct: bool = ...,
        filter: Optional[Any] = ...,
        **extra: Any
    ) -> None: ...

class Avg(Aggregate): ...
class Count(Aggregate): ...
class Max(Aggregate): ...
class Min(Aggregate): ...
class StdDev(Aggregate): ...
class Sum(Aggregate): ...
class Variance(Aggregate): ...



from typing import Sequence

from itertools import combinations

from ...classes.graph import Graph
from networkx import NetworkXError
from .community_utils import is_partition
from ...utils import not_implemented_for
from ...utils.decorators import argmap

__all__ = ["coverage", "modularity", "performance", "partition_quality"]

class NotAPartition(NetworkXError):
    def __init__(self, G: Graph, collection): ...

require_partition = ...

def intra_community_edges(G: Graph, partition): ...
def inter_community_edges(G: Graph, partition): ...
def inter_community_non_edges(G: Graph, partition): ...
@require_partition
def performance(G: Graph, partition: Sequence) -> float: ...
@require_partition
def coverage(G: Graph, partition: Sequence) -> float: ...
def modularity(G: Graph, communities, weight="weight", resolution=1) -> float: ...
@require_partition
def partition_quality(G: Graph, partition: Sequence): ...



from functools import wraps as wraps  # noqa: F401
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

from django.db.models.base import Model

def curry(_curried_func: Any, *args: Any, **kwargs: Any) -> Any: ...

_T = TypeVar("_T")

class cached_property(Generic[_T]):
    func: Callable[..., _T] = ...
    __doc__: Any = ...
    name: str = ...
    def __init__(self, func: Callable[..., _T], name: Optional[str] = ...) -> None: ...
    @overload
    def __get__(
        self, instance: None, cls: Type[Any] = ...
    ) -> "cached_property[_T]": ...
    @overload
    def __get__(self, instance: object, cls: Type[Any] = ...) -> _T: ...

class Promise: ...

def lazy(
    func: Union[Callable[..., Any], Type[str]], *resultclasses: Any
) -> Callable[..., Any]: ...
def lazystr(text: Any) -> Any: ...
def keep_lazy(*resultclasses: Any) -> Callable[..., Any]: ...
def keep_lazy_text(func: Callable[..., Any]) -> Callable[..., Any]: ...

empty: Any

def new_method_proxy(func: Any) -> Any: ...

class LazyObject:
    def __init__(self) -> None: ...
    __getattr__: Any = ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __reduce__(self) -> Tuple[Callable[..., Any], Tuple[Model]]: ...
    def __copy__(self) -> Any: ...
    def __deepcopy__(self, memo: Any) -> Any: ...
    __bytes__: Any = ...
    __bool__: Any = ...
    __dir__: Any = ...
    __class__: Any = ...
    __ne__: Any = ...
    __hash__: Any = ...
    __getitem__: Any = ...
    __setitem__: Any = ...
    __delitem__: Any = ...
    __iter__: Any = ...
    __len__: Any = ...
    __contains__: Any = ...

def unpickle_lazyobject(wrapped: Model) -> Model: ...

class SimpleLazyObject(LazyObject):
    def __init__(self, func: Callable[..., Any]) -> None: ...
    def __copy__(self) -> List[int]: ...
    def __deepcopy__(self, memo: Dict[Any, Any]) -> List[int]: ...

_PartitionMember = TypeVar("_PartitionMember")

def partition(
    predicate: Callable[..., Any], values: List[_PartitionMember]
) -> Tuple[List[_PartitionMember], List[_PartitionMember]]: ...



from abc import ABC, abstractmethod
from numpy import ndarray
from ..._loss.loss import HalfSquaredError as HalfSquaredError
from ..._typing import MatrixLike, ArrayLike, Float, Int
from .._linear_loss import LinearModelLoss
from scipy.sparse._csr import csr_matrix
from ...exceptions import ConvergenceWarning as ConvergenceWarning

# Author: Christian Lorentzen <lorentzen.ch@gmail.com>
# License: BSD 3 clause

import warnings

import numpy as np
import scipy.linalg
import scipy.optimize


class NewtonSolver(ABC):
    gradient_times_newton: float = ...
    use_fallback_lbfgs_solve: bool = ...
    iteration: int = ...
    converged: bool = ...
    raw_prediction: ndarray = ...
    loss_value_old: float = ...
    loss_value: float = ...
    gradient_old: ndarray = ...
    gradient: ndarray = ...
    coef_newton: ndarray = ...
    coef_old: ndarray = ...

    def __init__(
        self,
        *,
        coef: MatrixLike | ArrayLike,
        linear_loss: LinearModelLoss = ...,
        l2_reg_strength: Float = 0.0,
        tol: Float = 1e-4,
        max_iter: Int = 100,
        n_threads: Int = 1,
        verbose: int = 0,
    ) -> None:
        ...

    def setup(self, X: csr_matrix, y: ndarray, sample_weight: ndarray) -> None:
        ...

    @abstractmethod
    def update_gradient_hessian(self, X, y, sample_weight):
        ...

    @abstractmethod
    def inner_solve(self, X, y, sample_weight):
        ...

    def fallback_lbfgs_solve(self, X, y, sample_weight):
        ...

    def line_search(self, X: csr_matrix, y: ndarray, sample_weight: ndarray) -> None:
        ...

    def check_convergence(
        self, X: csr_matrix, y: ndarray, sample_weight: ndarray
    ) -> None:
        ...

    def finalize(self, X: csr_matrix, y: ndarray, sample_weight: ndarray) -> None:
        ...

    def solve(self, X: csr_matrix, y: ndarray, sample_weight: ndarray) -> ndarray:
        ...


class NewtonCholeskySolver(NewtonSolver):
    def setup(self, X: csr_matrix, y: ndarray, sample_weight: ndarray) -> None:
        ...

    def update_gradient_hessian(
        self, X: csr_matrix, y: ndarray, sample_weight: ndarray
    ) -> None:
        ...

    def inner_solve(self, X: csr_matrix, y: ndarray, sample_weight: ndarray) -> None:
        ...





# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division, print_function

import sys
import numpy as np
from time import sleep

from ..util.event import EmitterGroup, Event, WarningEmitter
from ..util.ptime import time
from ..util.dpi import get_dpi
from ..util import config as util_config, logger
from . import Application, use_app
from ..gloo.context import GLContext, set_current_canvas, forget_canvas
from ..gloo import FrameBuffer, RenderBuffer

from numpy.typing import ArrayLike
from typing import Mapping, Callable
from vispy.gloo.context import GLContext
from vispy.scene.widgets.widget import Widget
from vispy.app.application import Application
from vispy.util.keys import Key 

# todo: add functions for asking about current mouse/keyboard state
# todo: add hover enter/exit events
# todo: add focus events

class Canvas(object):
    def __init__(
        self,
        title: str = "VisPy canvas",
        size: tuple[float, float] = ...,
        position: tuple[float, float] | None = None,
        show: bool = False,
        autoswap: bool = True,
        app: str | Application | None = None,
        create_native: bool = True,
        vsync: bool = False,
        resizable: bool = True,
        decorate: bool = True,
        fullscreen: bool | int = False,
        config: Mapping | None = None,
        shared: None | Canvas | GLContext = None,
        keys: str | Mapping | None = None,
        parent: Widget | None = None,
        dpi: None | float = None,
        always_on_top: bool = False,
        px_scale: int = 1,
        backend_kwargs: Mapping | None = None,
    ): ...
    def create_native(self): ...
    def _set_keys(self, keys): ...
    @property
    def context(self): ...
    @property
    def app(self): ...
    @property
    def native(self): ...
    @property
    def dpi(self): ...
    @dpi.setter
    def dpi(self, dpi): ...
    def connect(self, fun: Callable): ...

    # ---------------------------------------------------------------- size ---
    @property
    def size(self): ...
    @size.setter
    def size(self, size): ...
    @property
    def physical_size(self): ...
    @property
    def pixel_scale(self): ...
    @property
    def fullscreen(self): ...
    @fullscreen.setter
    def fullscreen(self, fullscreen): ...

    # ------------------------------------------------------------ position ---
    @property
    def position(self): ...
    @position.setter
    def position(self, position): ...

    # --------------------------------------------------------------- title ---
    @property
    def title(self): ...
    @title.setter
    def title(self, title): ...

    # ----------------------------------------------------------------- fps ---
    @property
    def fps(self): ...
    def set_current(self, event=None): ...
    def swap_buffers(self, event=None): ...
    def show(self, visible: bool = True, run: bool = False): ...
    def update(self, event=None): ...
    def close(self): ...
    def _update_fps(self, event): ...
    def measure_fps(self, window: float = 1, callback: Callable | str = "%1.1f FPS"): ...

    # ---------------------------------------------------------------- misc ---
    def __repr__(self): ...
    def _repr_mimebundle_(self, *args, **kwargs): ...
    def _ipython_display_(self): ...
    def __enter__(self): ...
    def __exit__(self, type, value, traceback): ...
    def render(self, alpha: bool = True) -> ArrayLike: ...

# Event subclasses specific to the Canvas
class MouseEvent(Event):
    def __init__(
        self,
        type: str,
        pos: tuple[int, int] | None = None,
        button: None | int = None,
        buttons: tuple[int, ...] | None = None,
        modifiers: tuple[Key, ...] | None = None,
        delta: tuple[float, float] | None = None,
        last_event: MouseEvent | None = None,
        press_event: MouseEvent | None = None,
        **kwargs,
    ): ...
    @property
    def pos(self): ...
    @property
    def button(self): ...
    @property
    def buttons(self): ...
    @property
    def modifiers(self): ...
    @property
    def delta(self): ...
    @property
    def press_event(self): ...
    @property
    def last_event(self): ...
    @property
    def time(self): ...
    def _forget_last_event(self): ...
    @property
    def is_dragging(self): ...
    def drag_events(self): ...
    def trail(self): ...

class KeyEvent(Event):
    def __init__(self, type: str, key: Key | None = None, text: str = "", modifiers: tuple[Key, ...] | None = None, **kwargs): ...
    @property
    def key(self): ...
    @property
    def text(self): ...
    @property
    def modifiers(self): ...

class ResizeEvent(Event):
    def __init__(
        self, type: str, size: tuple[int, int] | None = None, physical_size: tuple[int, int] | None = None, **kwargs
    ): ...
    @property
    def size(self): ...
    @property
    def physical_size(self): ...

class DrawEvent(Event):
    def __init__(self, type: str, region: tuple[int, int, int, int] | None = None, **kwargs): ...
    @property
    def region(self): ...



"""
This type stub file was generated by pyright.
"""

import sys
from IPython.core.magic import Magics, line_magic, magics_class
from IPython.testing.skipdoctest import skip_doctest

"""Extra magics for terminal use."""
def get_pasted_lines(sentinel, l_input=..., quiet=...):
    """ Yield pasted lines until the user enters the given sentinel value.
    """
    ...

@magics_class
class TerminalMagics(Magics):
    def __init__(self, shell) -> None:
        ...
    
    def store_or_execute(self, block, name, store_history=...): # -> None:
        """ Execute a block, or store it in a variable, per the user's request.
        """
        ...
    
    def preclean_input(self, block):
        ...
    
    def rerun_pasted(self, name=...): # -> None:
        """ Rerun a previously pasted command.
        """
        ...
    
    @line_magic
    def autoindent(self, parameter_s=...): # -> None:
        """Toggle autoindent on/off (deprecated)"""
        ...
    
    @skip_doctest
    @line_magic
    def cpaste(self, parameter_s=...): # -> None:
        """Paste & execute a pre-formatted code block from clipboard.

        You must terminate the block with '--' (two minus-signs) or Ctrl-D
        alone on the line. You can also provide your own sentinel with '%paste
        -s %%' ('%%' is the new sentinel for this operation).

        The block is dedented prior to execution to enable execution of method
        definitions. '>' and '+' characters at the beginning of a line are
        ignored, to allow pasting directly from e-mails, diff files and
        doctests (the '...' continuation prompt is also stripped).  The
        executed block is also assigned to variable named 'pasted_block' for
        later editing with '%edit pasted_block'.

        You can also pass a variable name as an argument, e.g. '%cpaste foo'.
        This assigns the pasted block to variable 'foo' as string, without
        dedenting or executing it (preceding >>> and + is still stripped)

        '%cpaste -r' re-executes the block previously entered by cpaste.
        '%cpaste -q' suppresses any additional output messages.

        Do not be alarmed by garbled output on Windows (it's a readline bug).
        Just press enter and type -- (and press enter again) and the block
        will be what was just pasted.

        Shell escapes are not supported (yet).

        See Also
        --------
        paste : automatically pull code from clipboard.

        Examples
        --------
        ::

          In [8]: %cpaste
          Pasting code; enter '--' alone on the line to stop.
          :>>> a = ["world!", "Hello"]
          :>>> print(" ".join(sorted(a)))
          :--
          Hello world!

        ::
          In [8]: %cpaste
          Pasting code; enter '--' alone on the line to stop.
          :>>> %alias_magic t timeit
          :>>> %t -n1 pass
          :--
          Created `%t` as an alias for `%timeit`.
          Created `%%t` as an alias for `%%timeit`.
          354 ns ± 224 ns per loop (mean ± std. dev. of 7 runs, 1 loop each)
        """
        ...
    
    @line_magic
    def paste(self, parameter_s=...): # -> None:
        """Paste & execute a pre-formatted code block from clipboard.

        The text is pulled directly from the clipboard without user
        intervention and printed back on the screen before execution (unless
        the -q flag is given to force quiet mode).

        The block is dedented prior to execution to enable execution of method
        definitions. '>' and '+' characters at the beginning of a line are
        ignored, to allow pasting directly from e-mails, diff files and
        doctests (the '...' continuation prompt is also stripped).  The
        executed block is also assigned to variable named 'pasted_block' for
        later editing with '%edit pasted_block'.

        You can also pass a variable name as an argument, e.g. '%paste foo'.
        This assigns the pasted block to variable 'foo' as string, without
        executing it (preceding >>> and + is still stripped).

        Options:

          -r: re-executes the block previously entered by cpaste.

          -q: quiet mode: do not echo the pasted text back to the terminal.

        IPython statements (magics, shell escapes) are not supported (yet).

        See Also
        --------
        cpaste : manually paste code into terminal until you mark its end.
        """
        ...
    
    if sys.platform == 'win32':
        @line_magic
        def cls(self, s): # -> None:
            """Clear screen.
            """
            ...
        





import types
from datetime import date, datetime, time
from decimal import Decimal
from typing import (
    IO,
    Any,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)
from uuid import UUID

import psycopg2
from django.db.backends.postgresql.base import DatabaseWrapper
from psycopg2.extensions import Column
from typing_extensions import Literal

logger: Any

# Python types that can be adapted to SQL.
_Mixed = Union[None, bool, int, float, Decimal, str, bytes, datetime, UUID]
_SQLType = Union[_Mixed, Sequence[_Mixed], Mapping[str, _Mixed]]

class CursorWrapper:
    cursor: psycopg2.extensions.cursor = ...
    db: DatabaseWrapper = ...
    def __init__(
        self, cursor: psycopg2.extensions.cursor, db: DatabaseWrapper
    ) -> None: ...
    WRAP_ERROR_ATTRS: Any = ...
    def __iter__(self) -> Iterator[Tuple[Any, ...]]: ...
    def __next__(self) -> Tuple[Any, ...]: ...
    def __enter__(self) -> CursorWrapper: ...
    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        tb: Optional[types.TracebackType],
    ) -> None: ...
    def callproc(
        self,
        procname: str,
        params: Optional[Sequence[_SQLType]] = ...,
        kparams: Optional[Mapping[str, int]] = ...,
    ) -> None: ...
    def execute(
        self,
        sql: str,
        params: Optional[Union[Sequence[_SQLType], Mapping[str, _SQLType]]] = ...,
    ) -> None: ...
    def executemany(
        self,
        sql: str,
        param_list: Sequence[
            Optional[Union[Sequence[_SQLType], Mapping[str, _SQLType]]]
        ],
    ) -> None: ...
    # copied over from psycopg2 since Django uses __getattr__ to proxy calls
    @property
    def description(self) -> Optional[Tuple[Column, ...]]: ...
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    @property
    def connection(self) -> psycopg2.extensions.connection: ...
    @property
    def name(self) -> Optional[str]: ...
    scrollable: Optional[bool]
    withhold: bool
    def mogrify(
        self,
        operation: str,
        parameters: Optional[Union[Sequence[_SQLType], Mapping[str, _SQLType]]] = ...,
    ) -> bytes: ...
    def setinputsizes(self, sizes: int) -> None: ...
    def fetchone(self) -> Optional[Tuple[Any, ...]]: ...
    def fetchmany(self, size: int = ...) -> List[Tuple[Any, ...]]: ...
    def fetchall(self) -> List[Tuple[Any, ...]]: ...
    def scroll(
        self, value: int, mode: Union[Literal["relative"], Literal["absolute"]] = ...
    ) -> None: ...
    arraysize: int
    itersize: int
    @property
    def rowcount(self) -> int: ...
    @property
    def rownumber(self) -> int: ...
    @property
    def lastrowid(self) -> Optional[int]: ...
    @property
    def query(self) -> Optional[str]: ...
    @property
    def statusmessage(self) -> Optional[str]: ...
    def cast(self, oid: int, s: str) -> Any: ...
    tzinfo_factory: Any
    def nextset(self) -> None: ...
    def setoutputsize(self, size: int, column: int = ...) -> None: ...
    def copy_from(
        self,
        file: IO[str],
        table: str,
        sep: str = ...,
        null: str = ...,
        size: int = ...,
        columns: Optional[Iterable[str]] = ...,
    ) -> None: ...
    def copy_to(
        self,
        file: IO[str],
        table: str,
        sep: str = ...,
        null: str = ...,
        columns: Optional[str] = ...,
    ) -> None: ...
    def copy_expert(self, sql: str, file: IO[str], size: int = ...) -> None: ...

class CursorDebugWrapper(CursorWrapper):
    cursor: Any
    db: Any

def typecast_date(s: Optional[str]) -> Optional[date]: ...
def typecast_time(s: Optional[str]) -> Optional[time]: ...
def typecast_timestamp(s: Optional[str]) -> Optional[date]: ...
def rev_typecast_decimal(d: Decimal) -> str: ...
def split_identifier(identifier: str) -> Tuple[str, str]: ...
def truncate_name(
    identifier: str, length: Optional[int] = ..., hash_len: int = ...
) -> str: ...
def format_number(
    value: Optional[Decimal], max_digits: Optional[int], decimal_places: Optional[int]
) -> Optional[str]: ...
def strip_quotes(table_name: str) -> str: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.cluster._vq, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__test__: dict
def update_cluster_means() -> typing.Any:
    '\n    The update-step of K-means. Calculate the mean of observations in each\n    cluster.\n\n    Parameters\n    ----------\n    obs : ndarray\n        The observation matrix. Each row is an observation. Its dtype must be\n        float32 or float64.\n    labels : ndarray\n        The label of each observation. Must be an 1d array.\n    nc : int\n        The number of centroids.\n\n    Returns\n    -------\n    cb : ndarray\n        The new code book.\n    has_members : ndarray\n        A boolean array indicating which clusters have members.\n\n    Notes\n    -----\n    The empty clusters will be set to all zeros and the corresponding elements\n    in `has_members` will be `False`. The upper level function should decide\n    how to deal with them.\n    '
    ...

def vq() -> typing.Any:
    '\n    Vector quantization ndarray wrapper. Only support float32 and float64.\n\n    Parameters\n    ----------\n    obs : ndarray\n        The observation matrix. Each row is an observation.\n    codes : ndarray\n        The code book matrix.\n\n    Notes\n    -----\n    The observation matrix and code book matrix should have same ndim and\n    same number of columns (features). Only 1-dimensional and 2-dimensional\n    arrays are supported.\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




import numpy as np
from typing import Literal, Sequence
from ._typing import *
from .transforms import Bbox, Transform
from .path import Path
from ._enums import CapStyle, JoinStyle
from .backend_bases import MouseEvent, RendererBase
from .artist import Artist, allow_rasterization

class Patch(Artist):

    zorder: float = ...

    def __init__(
        self,
        edgecolor: Color = ...,
        facecolor: Color = ...,
        color: Color = ...,
        linewidth: float = ...,
        linestyle=...,
        antialiased: bool = ...,
        hatch=...,
        fill=...,
        capstyle: CapStyle = ...,
        joinstyle: JoinStyle = ...,
        **kwargs
    ) -> None: ...
    def get_verts(self): ...
    def contains(self, mouseevent: MouseEvent, radius: float = ...): ...
    def contains_point(self, point: Sequence[float], radius: float = ...) -> bool: ...
    def contains_points(self, points: ArrayLike, radius: float = ...) -> list[bool]: ...
    def update_from(self, other: Patch): ...
    def get_extents(self) -> Bbox: ...
    def get_transform(self) -> Transform: ...
    def get_data_transform(self) -> Transform: ...
    def get_patch_transform(self) -> Transform: ...
    def get_antialiased(self) -> bool: ...
    def get_edgecolor(self) -> Color: ...
    def get_facecolor(self) -> Color: ...
    def get_linewidth(self) -> float: ...
    def get_linestyle(self): ...
    def set_antialiased(self, aa: bool | None): ...
    def set_edgecolor(self, color: Color | None): ...
    def set_facecolor(self, color: Color | None): ...
    def set_color(self, c: Color): ...
    def set_alpha(self, alpha: Scalar | None): ...
    def set_linewidth(self, w: float | None): ...
    def set_linestyle(
        self,
        ls: Literal[
            "-",
            "solid",
            "--",
            "dashed",
            "-.",
            "dashdot",
            ":",
            "dotted",
            "None",
            "none",
            " ",
            "",
        ],
    ): ...
    def set_fill(self, b: bool): ...
    def get_fill(self) -> bool: ...
    fill = ...

    def set_capstyle(self, s: CapStyle | Literal["butt", "projecting", "round"]): ...
    def get_capstyle(self) -> CapStyle: ...
    def set_joinstyle(self, s: JoinStyle | Literal["miter", "round", "bevel"]): ...
    def get_joinstyle(self) -> JoinStyle | str: ...
    def set_hatch(
        self, hatch: Literal["/", "\\", "|", "-", "+", "x", "o", "O", ".", "*"]
    ): ...
    def get_hatch(self) -> str: ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def get_path(self) -> Path: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...

class Shadow(Patch):
    def __str__(self) -> str: ...
    def __init__(self, patch: Patch, ox: float, oy: float, **kwargs) -> None: ...
    def get_path(self) -> Path: ...
    def get_patch_transform(self) -> Transform: ...
    def draw(self, renderer: RendererBase): ...

class Rectangle(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        xy: Sequence[float],
        width: float,
        height: float,
        angle: float = 0,
        *,
        rotation_point: Literal["xy", "center"] | Sequence[float] = "xy",
        **kwargs
    ) -> None: ...
    def get_path(self) -> Path: ...
    def get_patch_transform(self): ...
    @property
    def rotation_point(self): ...
    @rotation_point.setter
    def rotation_point(self, value): ...
    def get_x(self) -> float: ...
    def get_y(self) -> float: ...
    def get_xy(self) -> tuple[float, float]: ...
    def get_corners(self) -> list[tuple[float, float]]: ...
    def get_center(self) -> tuple[float, float]: ...
    def get_width(self) -> float: ...
    def get_height(self) -> float: ...
    def get_angle(self) -> float: ...
    def set_x(self, x: float): ...
    def set_y(self, y: float): ...
    def set_angle(self, angle: float): ...
    def set_xy(self, xy: Sequence[float]): ...
    def set_width(self, w: float): ...
    def set_height(self, h: float): ...
    def set_bounds(self, *args): ...
    def get_bbox(self) -> Bbox: ...
    xy: tuple[float, float] = ...

class RegularPolygon(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        xy: Sequence[float],
        numVertices: int,
        radius: float = ...,
        orientation: float = ...,
        **kwargs
    ) -> None: ...
    def get_path(self) -> Path: ...
    def get_patch_transform(self) -> Transform: ...

class PathPatch(Patch):
    def __str__(self) -> str: ...
    def __init__(self, path: Path, **kwargs) -> None: ...
    def get_path(self) -> Path: ...
    def set_path(self, path: Path): ...

class StepPatch(PathPatch):
    def __init__(
        self,
        values: ArrayLike,
        edges: ArrayLike,
        *,
        orientation: Literal["vertical", "horizontal"] = "vertical",
        baseline: float | ArrayLike | None = 0,
        **kwargs
    ) -> None: ...
    def get_data(self): ...
    def set_data(
        self,
        values: ArrayLike | None = ...,
        edges: ArrayLike = ...,
        baseline: float | ArrayLike | None = ...,
    ): ...

class Polygon(Patch):
    def __str__(self) -> str: ...
    def __init__(self, xy: Sequence[float], closed: bool = ..., **kwargs) -> None: ...
    def get_path(self) -> Path: ...
    def get_closed(self) -> bool: ...
    def set_closed(self, closed: bool): ...
    def get_xy(self) -> np.ndarray: ...
    def set_xy(self, xy: ArrayLike): ...
    xy = ...

class Wedge(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        center: Sequence[float],
        r: float,
        theta1: float,
        theta2: float,
        width: float = ...,
        **kwargs
    ) -> None: ...
    def set_center(self, center: Sequence[float]): ...
    def set_radius(self, radius: float): ...
    def set_theta1(self, theta1: float): ...
    def set_theta2(self, theta2: float): ...
    def set_width(self, widt: float): ...
    def get_path(self) -> Path: ...

class Arrow(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self, x: float, y: float, dx: float, dy: float, width: float = ..., **kwargs
    ) -> None: ...
    def get_path(self) -> Path: ...
    def get_patch_transform(self) -> Transform: ...

class FancyArrow(Polygon):
    def __str__(self) -> str: ...
    def __init__(
        self,
        x: float,
        y: float,
        dx: float,
        dy: float,
        width: float = 0.001,
        length_includes_head: bool = False,
        head_width: float = ...,
        head_length: float | None = ...,
        shape: Literal["full", "left", "right"] = "full",
        overhang: float = 0,
        head_starts_at_zero: bool = False,
        **kwargs
    ) -> None: ...
    def set_data(
        self,
        *,
        x: float | None = None,
        y: float | None = None,
        dx: float | None = None,
        dy: float | None = None,
        width: float | None = None,
        head_width: float | None = None,
        head_length: float | None = None
    ): ...

class CirclePolygon(RegularPolygon):
    def __str__(self) -> str: ...
    def __init__(
        self, xy: Sequence[float], radius: float = ..., resolution=..., **kwargs
    ) -> None: ...

class Ellipse(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        xy: Sequence[float],
        width: float,
        height: float,
        angle: float = 0,
        **kwargs
    ) -> None: ...
    def get_path(self) -> Path: ...
    def get_patch_transform(self) -> Transform: ...
    def set_center(self, xy: Sequence[float]): ...
    def get_center(self) -> tuple[float, float]: ...
    center = ...
    def set_width(self, width: float): ...
    def get_width(self) -> float: ...
    width = ...
    def set_height(self, height: float): ...
    def get_height(self) -> float: ...
    height = ...
    def set_angle(self, angle: float): ...
    def get_angle(self) -> float: ...
    angle = ...
    def get_corners(self): ...

class Annulus(Patch):
    def __init__(
        self,
        xy: Sequence[float],
        r: float | Sequence[float],
        width: float,
        angle: float = 0,
        **kwargs
    ) -> None: ...
    def __str__(self) -> str: ...
    def set_center(self, xy: Sequence[float]): ...
    def get_center(self) -> tuple[float, float]: ...
    center = ...
    def set_width(self, width: float): ...
    def get_width(self) -> float: ...
    width = ...
    def set_angle(self, angle: float): ...
    def get_angle(self) -> float: ...
    angle = ...
    def set_semimajor(self, a: float): ...
    def set_semiminor(self, b: float): ...
    def set_radii(self, r: float): ...
    def get_radii(self) -> float: ...
    radii = ...
    def get_path(self) -> Path: ...

class Circle(Ellipse):
    def __str__(self) -> str: ...
    def __init__(self, xy: Sequence[float], radius: float = ..., **kwargs) -> None: ...
    def set_radius(self, radius: float): ...
    def get_radius(self) -> float: ...
    radius: float = ...

class Arc(Ellipse):
    def __str__(self) -> str: ...
    def __init__(
        self,
        xy: Sequence[float],
        width: float,
        height: float,
        angle: float = 0,
        theta1: float = 0,
        theta2: float = 360,
        **kwargs
    ) -> None: ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...

def bbox_artist(artist, renderer: RendererBase, props: dict = ..., fill=...): ...
def draw_bbox(
    bbox, renderer: RendererBase, color: Color = ..., trans: Transform = ...
): ...

class _Style:
    def __new__(cls, stylename: str, **kwargs): ...
    @classmethod
    def get_styles(cls): ...
    @classmethod
    def pprint_styles(cls): ...
    @classmethod
    def register(cls, name: str, style: _Style): ...

class BoxStyle(_Style):
    class Square:
        def __init__(self, pad: float = 0.3) -> None: ...
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class Circle:
        def __init__(self, pad: float = 0.3) -> None: ...
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class LArrow:
        def __init__(self, pad: float = 0.3) -> None: ...
        def __call__(self, x0, y0, width, height, mutation_size): ...

    class RArrow(LArrow):
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class DArrow:
        def __init__(self, pad: float = 0.3) -> None: ...
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class Round:
        def __init__(self, pad=0.3, rounding_size: float = ...) -> None: ...
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class Round4:
        def __init__(self, pad: float = 0.3, rounding_size: float = ...) -> None: ...
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class Sawtooth:
        def __init__(self, pad: float = 0.3, tooth_size: float = ...) -> None: ...
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

    class Roundtooth(Sawtooth):
        def __call__(
            self, x0: float, y0: float, width: float, height: float, mutation_size
        ): ...

class ConnectionStyle(_Style):
    class _Base:
        class SimpleEvent:
            def __init__(self, xy: Sequence[float]) -> None: ...

        def __call__(
            self, posA, posB, shrinkA=..., shrinkB=..., patchA=..., patchB=...
        ): ...

    class Arc3(_Base):
        def __init__(self, rad=...) -> None: ...
        def connect(self, posA, posB): ...

    class Angle3(_Base):
        def __init__(self, angleA: float = ..., angleB: float = ...) -> None: ...
        def connect(self, posA, posB): ...

    class Angle(_Base):
        def __init__(
            self, angleA: float = ..., angleB: float = ..., rad: float = ...
        ) -> None: ...
        def connect(self, posA, posB): ...

    class Arc(_Base):
        def __init__(
            self,
            angleA: float = ...,
            angleB: float = ...,
            armA: float = ...,
            armB: float = ...,
            rad: float = ...,
        ) -> None: ...
        def connect(self, posA, posB): ...

    class Bar(_Base):
        def __init__(
            self,
            armA: float = ...,
            armB: float = ...,
            fraction: float = ...,
            angle: float = ...,
        ) -> None: ...
        def connect(self, posA, posB): ...

class ArrowStyle(_Style):
    class _Base:
        @staticmethod
        def ensure_quadratic_bezier(path: Path): ...
        def transmute(self, path: Path, mutation_size, linewidth: float): ...
        def __call__(
            self, path: Path, mutation_size, linewidth: float, aspect_ratio: float = ...
        ): ...

    class _Curve(_Base):

        beginarrow = ...
        arrow = ...
        fillbegin = ...
        def __init__(
            self,
            head_length: float = ...,
            head_width: float = ...,
            widthA: float = ...,
            widthB: float = ...,
            lengthA: float = ...,
            lengthB: float = ...,
            angleA: float = ...,
            angleB: float = ...,
            scaleA: float = ...,
            scaleB: float = ...,
        ) -> None: ...
        def transmute(self, path, mutation_size, linewidth): ...

    class Curve(_Curve):
        def __init__(self) -> None: ...

    class CurveA(_Curve):

        arrow = ...

    class CurveB(_Curve):

        arrow = ...

    class CurveAB(_Curve):

        arrow = ...

    class CurveFilledA(_Curve):

        arrow = ...

    class CurveFilledB(_Curve):

        arrow = ...

    class CurveFilledAB(_Curve):

        arrow = ...

    class BracketA(_Curve):

        arrow = ...
        def __init__(
            self, widthA: float = ..., lengthA: float = ..., angleA: float = ...
        ) -> None: ...

    class BracketB(_Curve):

        arrow = ...
        def __init__(
            self, widthB: float = ..., lengthB: float = ..., angleB: float = ...
        ) -> None: ...

    class BracketAB(_Curve):

        arrow = ...
        def __init__(
            self,
            widthA: float = ...,
            lengthA: float = ...,
            angleA: float = ...,
            widthB: float = ...,
            lengthB: float = ...,
            angleB: float = ...,
        ) -> None: ...

    class BarAB(_Curve):

        arrow = ...
        def __init__(
            self,
            widthA: float = ...,
            angleA: float = ...,
            widthB: float = ...,
            angleB: float = ...,
        ) -> None: ...

    class BracketCurve(_Curve):

        arrow = ...
        def __init__(
            self, widthA: float = ..., lengthA: float = ..., angleA: float = ...
        ) -> None: ...

    class CurveBracket(_Curve):

        arrow = ...
        def __init__(
            self, widthB: float = ..., lengthB: float = ..., angleB: float = ...
        ) -> None: ...

    class Simple(_Base):
        def __init__(
            self,
            head_length: float = ...,
            head_width: float = ...,
            tail_width: float = ...,
        ) -> None: ...
        def transmute(self, path: Path, mutation_size, linewidth: float): ...

    class Fancy(_Base):
        def __init__(
            self,
            head_length: float = ...,
            head_width: float = ...,
            tail_width: float = ...,
        ) -> None: ...
        def transmute(self, path: Path, mutation_size, linewidth: float): ...

    class Wedge(_Base):
        def __init__(
            self, tail_width: float = ..., shrink_factor: float = ...
        ) -> None: ...
        def transmute(self, path: Path, mutation_size, linewidth: float): ...

class FancyBboxPatch(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        xy: Sequence[float],
        width: float,
        height: float,
        boxstyle: str | BoxStyle = ...,
        bbox_transmuter=...,
        mutation_scale: float = ...,
        mutation_aspect: float = ...,
        **kwargs
    ) -> None: ...
    def set_boxstyle(self, boxstyle: str | BoxStyle = ..., **kwargs): ...
    def set_mutation_scale(self, scale: float): ...
    def get_mutation_scale(self): ...
    def set_mutation_aspect(self, aspect: float): ...
    def get_mutation_aspect(self) -> float: ...
    def get_boxstyle(self) -> str | BoxStyle: ...
    def get_path(self) -> Path: ...
    def get_x(self) -> float: ...
    def get_y(self) -> float: ...
    def get_width(self) -> float: ...
    def get_height(self) -> float: ...
    def set_x(self, x: float): ...
    def set_y(self, y: float): ...
    def set_width(self, w: float): ...
    def set_height(self, h: float): ...
    def set_bounds(self, *args): ...
    def get_bbox(self) -> Bbox: ...

class FancyArrowPatch(Patch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        posA: Sequence[float] = ...,
        posB: Sequence[float] = ...,
        path: Path = ...,
        arrowstyle: str | ArrowStyle = ...,
        connectionstyle: str | ConnectionStyle = ...,
        patchA: Patch = ...,
        patchB: Patch = ...,
        shrinkA: float = ...,
        shrinkB: float = ...,
        mutation_scale: float = ...,
        mutation_aspect: float = ...,
        **kwargs
    ) -> None: ...
    def set_positions(
        self, posA: None | Sequence[float], posB: None | Sequence[Sequence]
    ): ...
    def set_patchA(self, patchA: Patch): ...
    def set_patchB(self, patchB: Patch): ...
    def set_connectionstyle(
        self, connectionstyle: str | ConnectionStyle | None, **kwargs
    ): ...
    def get_connectionstyle(self) -> ConnectionStyle: ...
    def set_arrowstyle(self, arrowstyle: None | ArrowStyle | str = ..., **kwargs): ...
    def get_arrowstyle(self) -> ArrowStyle: ...
    def set_mutation_scale(self, scale: float): ...
    def get_mutation_scale(self) -> float: ...
    def set_mutation_aspect(self, aspect: float): ...
    def get_mutation_aspect(self) -> float: ...
    def get_path(self) -> Path: ...
    get_path_in_displaycoord = ...
    def draw(self, renderer: RendererBase): ...

class ConnectionPatch(FancyArrowPatch):
    def __str__(self) -> str: ...
    def __init__(
        self,
        xyA: Sequence[float],
        xyB: Sequence[float],
        coordsA,
        coordsB=...,
        axesA=...,
        axesB=...,
        arrowstyle: str | ArrowStyle = ...,
        connectionstyle: str | ConnectionStyle = ...,
        patchA: Patch = ...,
        patchB: Patch = ...,
        shrinkA: float = ...,
        shrinkB: float = ...,
        mutation_scale: float = ...,
        mutation_aspect: float = ...,
        clip_on: bool = ...,
        **kwargs
    ) -> None: ...
    def set_annotation_clip(self, b: bool | None): ...
    def get_annotation_clip(self) -> bool: ...
    def draw(self, renderer): ...



import warnings
from collections import defaultdict

from ..classes.graph import Graph
from ..utils import open_file

__all__ = [
    "write_graphml",
    "read_graphml",
    "generate_graphml",
    "write_graphml_xml",
    "write_graphml_lxml",
    "parse_graphml",
    "GraphMLWriter",
    "GraphMLReader",
]

def write_graphml_xml(
    G: Graph,
    path,
    encoding="utf-8",
    prettyprint=True,
    infer_numeric_types: bool = False,
    named_key_ids=False,
    edge_id_from_attribute=None,
): ...
def write_graphml_lxml(
    G: Graph,
    path,
    encoding="utf-8",
    prettyprint=True,
    infer_numeric_types: bool = False,
    named_key_ids=False,
    edge_id_from_attribute=None,
): ...
def generate_graphml(
    G: Graph,
    encoding="utf-8",
    prettyprint=True,
    named_key_ids=False,
    edge_id_from_attribute=None,
): ...
def read_graphml(path, node_type=..., edge_key_type=..., force_multigraph=False): ...
def parse_graphml(
    graphml_string: str, node_type=..., edge_key_type=..., force_multigraph=False
): ...

class GraphML:
    NS_GRAPHML: str = ...
    NS_XSI: str = ...
    # xmlns:y="http://www.yworks.com/xml/graphml"
    NS_Y: str = ...
    SCHEMALOCATION = ...

    def construct_types(self): ...

    # This page says that data types in GraphML follow Java(TM).
    #  http://graphml.graphdrawing.org/primer/graphml-primer.html#AttributesDefinition
    # true and false are the only boolean literals:
    #  http://en.wikibooks.org/wiki/Java_Programming/Literals#Boolean_Literals
    convert_bool: dict = ...

    def get_xml_type(self, key): ...

class GraphMLWriter(GraphML):
    def __init__(
        self,
        graph=None,
        encoding="utf-8",
        prettyprint=True,
        infer_numeric_types=False,
        named_key_ids=False,
        edge_id_from_attribute=None,
    ): ...
    def __str__(self): ...
    def attr_type(self, name, scope, value): ...
    def get_key(self, name, attr_type, scope, default): ...
    def add_data(self, name, element_type, value, scope="all", default=None): ...
    def add_attributes(self, scope, xml_obj, data, default): ...
    def add_nodes(self, G: Graph, graph_element): ...
    def add_edges(self, G: Graph, graph_element): ...
    def add_graph_element(self, G): ...
    def add_graphs(self, graph_list): ...
    def dump(self, stream): ...
    def indent(self, elem, level=0): ...

class IncrementalElement:
    def __init__(self, xml, prettyprint): ...
    def append(self, element): ...

class GraphMLWriterLxml(GraphMLWriter):
    def __init__(
        self,
        path,
        graph=None,
        encoding="utf-8",
        prettyprint=True,
        infer_numeric_types=False,
        named_key_ids=False,
        edge_id_from_attribute=None,
    ): ...
    def add_graph_element(self, G): ...
    def add_attributes(self, scope, xml_obj, data, default): ...
    def __str__(self): ...
    def dump(self): ...

# default is lxml is present.
write_graphml = ...

class GraphMLReader(GraphML):
    def __init__(self, node_type=..., edge_key_type=..., force_multigraph=False): ...
    def __call__(self, path=None, string=None): ...
    def make_graph(self, graph_xml, graphml_keys, defaults, G=None): ...
    def add_node(self, G: Graph, node_xml, graphml_keys, defaults): ...
    def add_edge(self, G: Graph, edge_element, graphml_keys): ...
    def decode_data_elements(self, graphml_keys, obj_xml): ...
    def find_graphml_keys(self, graph_element): ...



"""
This type stub file was generated by pyright.
"""

"""
Utilities for getting information about IPython and the system it's running in.
"""
def pkg_commit_hash(pkg_path):
    """Get short form of commit hash given directory `pkg_path`

    We get the commit hash from (in order of preference):

    * IPython.utils._sysinfo.commit
    * git output, if we are in a git repository

    If these fail, we return a not-found placeholder tuple

    Parameters
    ----------
    pkg_path : str
        directory containing package
        only used for getting commit from active repo

    Returns
    -------
    hash_from : str
        Where we got the hash from - description
    hash_str : str
        short form of hash
    """
    ...

def pkg_info(pkg_path):
    """Return dict describing the context of this package

    Parameters
    ----------
    pkg_path : str
        path containing __init__.py for package

    Returns
    -------
    context : dict
        with named parameters of interest
    """
    ...

def get_sys_info():
    """Return useful information about IPython and the system, as a dict."""
    ...

def sys_info():
    """Return useful information about IPython and the system, as a string.

    Examples
    --------
    ::
    
        In [2]: print(sys_info())
        {'commit_hash': '144fdae',      # random
         'commit_source': 'repository',
         'ipython_path': '/home/fperez/usr/lib/python2.6/site-packages/IPython',
         'ipython_version': '0.11.dev',
         'os_name': 'posix',
         'platform': 'Linux-2.6.35-22-generic-i686-with-Ubuntu-10.10-maverick',
         'sys_executable': '/usr/bin/python',
         'sys_platform': 'linux2',
         'sys_version': '2.6.6 (r266:84292, Sep 15 2010, 15:52:39) \\n[GCC 4.4.5]'}
    """
    ...

def num_cpus():
    """DEPRECATED

    Return the effective number of CPUs in the system as an integer.

    This cross-platform function makes an attempt at finding the total number of
    available CPUs in the system, as returned by various underlying system and
    python calls.

    If it can't find a sensible answer, it returns 1 (though an error *may* make
    it return a large positive number that's actually incorrect).
    """
    ...




__all__ = [
    "NotFittedError",
    "ConvergenceWarning",
    "DataConversionWarning",
    "DataDimensionalityWarning",
    "EfficiencyWarning",
    "FitFailedWarning",
    "SkipTestWarning",
    "UndefinedMetricWarning",
    "PositiveSpectrumWarning",
]


class NotFittedError(ValueError, AttributeError):
    ...


class ConvergenceWarning(UserWarning):
    ...


class DataConversionWarning(UserWarning):
    ...


class DataDimensionalityWarning(UserWarning):
    ...


class EfficiencyWarning(UserWarning):
    ...


class FitFailedWarning(RuntimeWarning):
    ...


class SkipTestWarning(UserWarning):
    ...


class UndefinedMetricWarning(UserWarning):
    ...


class PositiveSpectrumWarning(UserWarning):
    ...
